@@ -15,9 +15,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
  *
- * $Id: plugin.c,v 1.31 2005/08/11 19:00:21 dsalt Exp $
+ * $Id: plugin.c,v 1.33 2006-02-21 19:06:11 dsalt Exp $
  *
  * xine plugin for mozilla
  *
@@ -157,14 +157,14 @@ char *NPP_GetMIMEDescription(void) {
          "video/x-msvideo: avi: AVI animation;"
          "application/x-mplayer2: asf,asx,asp: mplayer2;"
          "video/x-ms-asf-plugin: asf,asx,asp: mms animation;"
-         "audio/x-pn-realaudio-plugin: rpm: Real audio;"
+         /*"audio/x-pn-realaudio-plugin: rpm: Real audio;"*/
          "audio/x-ogg: ogg,ogm: OGG Media;"
          "audio/x-scpls: pls: MPEG audio"
 	 ;
 }
 
-NPError NPP_GetValue(void *future, NPPVariable variable, void *value){
-
+NPError NPP_GetValue (NPP instance, NPPVariable variable, void *value)
+{
   NPError err = NPERR_NO_ERROR;
   xprintf("NPP_GetValue: variable=%d\n", variable);
 
@@ -328,7 +328,10 @@ static void launch_gxine (plugin_instance_t *this) {
 static void got_url (const char *url_) {
 
   if (strstr (url_, ":/"))
+  {
+    free (globals.url);
     globals.url = strdup (url_);
+  }
   else
     xprintf ("got_url: ignoring this url (%s) because it is a relative one.\n",
 	     url_);
@@ -571,6 +574,12 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save) {
 
   }
 
+  if (globals.url)
+  {
+    free (globals.url);
+    globals.url = NULL;
+  }
+
   globals.gxine_started = FALSE;
 
   xprintf("NPP_Destroy: closed.\n");
