@@ -163,7 +163,7 @@ int engine_exec_obj (const char *cmd, se_o_t *obj, se_print_cb_t cb,
     JSInt32 i;
     JSFloat64 d;
     JSBool b;
-  } num; 
+  } num;
   char *str;
 
 #ifdef LOG
@@ -184,14 +184,16 @@ int engine_exec_obj (const char *cmd, se_o_t *obj, se_print_cb_t cb,
   if (verbosity)
     g_print (_("engine: > executing ‘%s’...\n"), cmd);
 
-  if (!cb) {
+  if (!cb)
+  {
     cb = print_cb;
     cb_data = NULL;
   }
 
   se_eval_ext (gse, cmd, obj, cb, cb_data, ecb, src);
 
-  if ( (str = se_result_str (gse)) ) {
+  if ((str = se_result_str (gse)))
+  {
     str = g_strdup_printf (_("result: %s"), str);
     cb (cb_data, "%s", str);
     free (str);
@@ -211,7 +213,7 @@ int engine_exec_obj (const char *cmd, se_o_t *obj, se_print_cb_t cb,
   else if (se_result_bool (gse, &num.b))
   {
     char str[32];
-    snprintf (str, sizeof (str), _("result: %s"), 
+    snprintf (str, sizeof (str), _("result: %s"),
 	      num.b == JS_TRUE ? _("true") : _("false"));
     cb (cb_data, "%s", str);
   }
@@ -295,25 +297,39 @@ void engine_queue_push (const char *command, se_o_t *obj, se_print_cb_t cb,
 static gboolean js_queue_cb (GtkWidget *widget, GdkEventClient *event,
 			    gpointer data)
 {
+  /* js_lock must be recursive since, if the engine is busy, this function
+   * WILL be re-entered from the same thread with the lock already held.
+   */
+  static pthread_mutex_t js_lock = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
   exec_t *js;
+
+  pthread_mutex_lock (&js_lock);
   while ((js = g_async_queue_try_pop (js_queue)))
   {
-    /* spin; play_exec gets this & the GDK lock in reverse order */ 
-    while (pthread_mutex_trylock (&widgets_update_lock))
-      sched_yield ();
+    /* This function is entered with the GDK lock held. This is bad, from
+     * gxine's POV since it needs to sync on widgets_update_lock (play_exec
+     * holds onto it for a while), and play_exec claims that *then* the GDK
+     * lock, which it needs to release from time to time. So we have to
+     * release the GDK lock for a short time.
+     * This _really_ needs a better solution...
+     */
+    gdk_threads_leave ();
+    pthread_mutex_lock (&widgets_update_lock);
     pthread_mutex_unlock (&widgets_update_lock); /* JS 'play()' needs it */
+    gdk_threads_enter ();
     engine_exec_obj (js->cmd, js->obj, js->cb, js->cb_data, js->ecb, js->src);
     free (js->cmd);
     free (js->src);
     free (js);
   }
+  pthread_mutex_unlock (&js_lock);
   return TRUE;
 }
 
 static char *
 load_user_startup_script (void)
 {
-  char *file = g_strconcat (g_get_home_dir (), "/.gxine/startup", NULL);
+  char *file = get_config_filename (FILE_STARTUP);
   char *text = read_entire_file (file, NULL);
   if (!text)
   {
@@ -367,7 +383,7 @@ se_startup_response_cb (GtkWidget *widget, int response, gpointer data)
   {
   case 1:
     {
-      char *file = g_strconcat (g_get_home_dir (), "/.gxine/startup", NULL);
+      char *file = get_config_filename (FILE_STARTUP);
       char *tmp = read_entire_file (file, NULL);
       if (!tmp && errno != ENOENT)
 	display_error (FROM_GXINE, _("Couldn't load startup script"), "%s: %s",
@@ -412,14 +428,14 @@ void engine_init (void)
 {
   char *cfgfilename;
   GtkWidget *w;
-  
+
   /*
    * init the global xine engine
    */
 
   xine = xine_new ();
 
-  cfgfilename = g_strconcat (g_get_home_dir(), "/.gxine/config", NULL);
+  cfgfilename = get_config_filename (FILE_CONFIG);
   xine_config_load (xine, cfgfilename);
   free (cfgfilename);
   xine_engine_set_param (xine, XINE_ENGINE_PARAM_VERBOSITY, verbosity);
@@ -428,7 +444,7 @@ void engine_init (void)
   ui_preferences_register (xine);
 
   /* init the javascript engine */
-  
+
   gse = se_new ();
 
   static const se_f_def_t defs[] = {
@@ -441,7 +457,7 @@ void engine_init (void)
   /*
    * create the startup commands editor window
    */
-   
+
   se_startup_window = gtk_dialog_new_with_buttons (_("Startup script"), NULL, 0,
 				GTK_STOCK_REVERT_TO_SAVED, 1,
 				GTK_STOCK_UNDO, GTK_RESPONSE_REJECT,
@@ -488,7 +504,7 @@ engine_startup_cb (gpointer data)
 void
 engine_startup_script (void)
 {
-  gchar *file, *name = g_strconcat (confdir, "/startup", NULL);
+  gchar *file, *name = g_build_filename (confdir, "startup", NULL);
   GtkTextIter pos;
 
   file = read_entire_file (name, NULL);
@@ -525,7 +541,7 @@ save_startup_script (void)
   if (!modified)
     return;
 
-  char *fname = g_strconcat (g_get_home_dir(), "/.gxine/startup", NULL);
+  char *fname = get_config_filename (FILE_STARTUP);
   FILE *f = open_write (fname, _("Failed to save startup script"));
 
   if (f)
