@@ -13,9 +13,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: gtkflipbutton.h,v 1.3 2005/03/12 00:19:11 dsalt Exp $
+ * $Id: gtkflipbutton.h,v 1.4 2006-02-21 19:06:11 dsalt Exp $
  *
  * a GTK toggle-button-like widget
  */
@@ -62,6 +62,8 @@ extern "C"
   GtkWidget *gtk_flip_button_new (GtkWidget *, GtkWidget *);
   GtkWidget *gtk_flip_button_new_from_stock (const char *, const char *,
   					     GtkIconSize);
+  GtkWidget *gtk_flip_button_new_from_icon_names (const char *, const char *,
+						  GtkIconSize);
 
 #ifdef __cplusplus
 }
