@@ -1,5 +1,5 @@
-/* 
- * Copyright (C) 2003-2005 the xine project
+/*
+ * Copyright (C) 2003-2006 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: key_events.c,v 1.91 2005/11/10 22:56:50 dsalt Exp $
+ * $Id: key_events.c,v 1.104 2006/04/08 21:34:49 dsalt Exp $
  *
  * key event handler and keymap editor
  */
@@ -56,19 +56,22 @@ GtkWidget *keypad;
 static gboolean        list_visible = FALSE, edit_visible = FALSE,
 		       is_new_binding = FALSE;
 static GtkListStore   *kb_store;
+static GtkTreeModel   *kb_model;
 static GtkWidget      *kb_bindings_list_dlg, *tree_view,
 		      *kb_binding_edit_dlg, *kb_binding_desc,
 		      *kb_binding_command, *kb_binding_key, *kb_binding_quote;
 static GtkTextBuffer  *kb_binding_command_buf;
-static GtkAccelGroup  *kb_binding_edit_accel;
 
 typedef struct {
   const gchar *desc, *cmd;
-  guint        keyval, state;
+  guint        keyval;
+  guint        state;
   const gchar *keyname;
+  gboolean     std;
 } key_binding_t;
 
 static key_binding_t editkey;
+static const char *editdesc;
 static const key_binding_t null_binding = { 0, 0, GDK_VoidSymbol, 0 };
 static key_binding_t default_binding = { N_("New binding"), "", GDK_VoidSymbol, 0 };
 
@@ -77,11 +80,16 @@ static GtkTreeIter catch_key_iter;
 static void delete_key_binding (GtkTreeIter *);
 static gchar *kb2str (key_binding_t *kb);
 
+#define COLUMN_DESC 0
+#define COLUMN_KEY  1
+#define COLUMN_DATA 2
+
 static const key_binding_t default_bindings[] = {
   { N_("Play"), "vdr ('PLAY') || play ();", GDK_Return, 0 },
   { N_("Exit gxine"), "exit ();", GDK_q, 0 },
   { N_("Pause"), "if (!vdr ('PAUSE') && !is_live_stream ()) pause ();", GDK_space, 0 },
   { N_("Stop"), "vdr ('STOP') || stop ();", 0, 0 },
+  { N_("Eject"), "eject ();", 0, 0 },
   { N_("Windowed mode"), "vo_fullscreen.v = false;", GDK_Escape, 0 },
   { N_("Fullscreen mode"), "vo_fullscreen.toggle ();", GDK_f, 0 },
   { N_("Aspect ratio"), "++vo_aspect.v;", GDK_a, 0 },
@@ -95,6 +103,7 @@ static const key_binding_t default_bindings[] = {
   { N_("Left"), "input_left ();", GDK_KP_4, 0 },
   { N_("Right"), "input_right ();", GDK_KP_6, 0 },
   { N_("Select/OK"), "input_select ();", GDK_KP_Enter, 0 },
+  /* Menus: 1 is the DVD root menu. All have the VDR function given in []. */
   { N_("Menu 1 [main]"), "input_menu1 ();", GDK_F1, 0 },
   { N_("Menu 2 [schedule]"), "vdr ('SCHEDULE') || input_menu2 ();", GDK_F2, 0 },
   { N_("Menu 3 [channels]"), "vdr ('CHANNELS') || input_menu3 ();", GDK_F3, 0 },
@@ -116,14 +125,21 @@ static const key_binding_t default_bindings[] = {
   { N_("8 / Play last 20%"), "vdr ('8') || play (80, 0);", GDK_8, 0 },
   { N_("9 / Play last 10%"), "vdr ('9') || play (90, 0);", GDK_9, 0 },
   { N_("0 / Play from start"), "vdr ('0') || play (0, 0);", GDK_0, 0 },
+  /* (VDR) The four coloured buttons on many remote controls */
   { N_("Red"), "vdr ('RED');", GDK_r, 0 },
   { N_("Green"), "vdr ('GREEN');", GDK_g, 0 },
   { N_("Yellow"), "vdr ('YELLOW');", GDK_y, 0 },
   { N_("Blue"), "vdr ('BLUE');", GDK_b, 0 },
+  /* (VDR) Start recording the current channel */
   { N_("Record"), "vdr ('RECORD');", GDK_r, GDK_CONTROL_MASK },
+  /* (VDR) Power or Standby button */
   { N_("Power"), "vdr ('POWER');", 0, 0 },
+  /* (VDR) Go back to a previous menu */
   { N_("Back"), "vdr ('BACK');", GDK_BackSpace, 0 },
+  /* (VDR) Audio menu */
   { N_("Audio"), "vdr ('AUDIO');", 0, 0 },
+  /* (VDR) Info display */
+  { N_("Info"), "vdr ('INFO');", 0, 0 },
   { N_("Playlist next"), "playlist_play (playlist_get_item()+1);", GDK_Page_Down, 0 },
   { N_("Playlist previous"), "playlist_play (playlist_get_item()-1);", GDK_Page_Up, 0 },
   { N_("Zoom in"), "vo_zoom.v += 5;", GDK_Z, GDK_SHIFT_MASK },
@@ -135,13 +151,17 @@ static const key_binding_t default_bindings[] = {
   { 0 }
 };
 
+static inline const char *getdesc (const key_binding_t *binding)
+{
+   return binding->std ? gettext (binding->desc) : binding->desc;
+}
 
 static key_binding_t *lookup_binding (GtkTreeIter *iter)
 {
   key_binding_t *k;
   GValue v = {0};
 
-  gtk_tree_model_get_value (GTK_TREE_MODEL (kb_store), iter, 2, &v);
+  gtk_tree_model_get_value (kb_model, iter, COLUMN_DATA, &v);
   k = g_value_peek_pointer (&v);
   g_value_unset (&v);
   return k;
@@ -154,18 +174,23 @@ gboolean keypress_cb (GtkWidget *win, GdkEventKey *event, gpointer data)
   guint keyval = gdk_keyval_to_lower (event->keyval);
   guint state = event->state & GXINE_MODIFIER_MASK;
 
-  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), &iter)) {
-    do {
+  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), &iter))
+  {
+    do
+    {
       key_binding_t *key_binding = lookup_binding (&iter);
-      if (key_binding->keyval == keyval	&& key_binding->state == state)
+      if (key_binding->state == state &&
+	  (key_binding->keyval == keyval ||
+	   key_binding->keyval == ~event->hardware_keycode))
       {
         char *binding = kb2str (key_binding);
         asreprintf (&binding, _("key binding %s"), binding);
-	engine_exec (key_binding->cmd, NULL, NULL, binding);
+	/* defer execution of commands due to keypresses during startup */
+	engine_queue_push (key_binding->cmd, NULL, NULL, NULL, NULL, binding);
 	free (binding);
 	return TRUE;
       }
-    } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (kb_store), &iter));
+    } while (gtk_tree_model_iter_next (kb_model, &iter));
   }
 
   /* Menu shortcuts */
@@ -191,8 +216,11 @@ gboolean keypress_cb (GtkWidget *win, GdkEventKey *event, gpointer data)
     switch (keyval)
     {
     case GDK_F10:
-      if (menubar && GTK_WIDGET_VISIBLE (gtk_widget_get_toplevel (menubar)))
-	break;
+      {
+	GtkWidget *toplevel = gtk_widget_get_toplevel (menubar);
+	if (menubar && GTK_WIDGET_VISIBLE (toplevel == app ? menubar : toplevel))
+	  break;
+      }
     case GDK_Menu:
       gtk_menu_popup (GTK_MENU(popup_menu), NULL, NULL, NULL, NULL, 0,
                       gtk_get_current_event_time ());
@@ -211,7 +239,7 @@ gboolean buttonpress_cb (GtkWidget *win, GdkEventButton *event, gpointer data)
     switch (event->button)
     {
     case 2:
-      if (gtk_video_is_fullscreen (GTK_VIDEO (gtv)))
+      if (gtk_video_is_fullscreen ((GtkVideo *)gtv))
 	gtk_action_activate (GTK_ACTION(action_items.fs_toolbar));
       else
 	gtk_action_activate (GTK_ACTION(action_items.wm_toolbar));
@@ -250,7 +278,8 @@ static void close_edit_window (void)
 }
 
 
-static gboolean close_cb (GtkWidget* widget, gpointer data) {
+static gboolean close_cb (GtkWidget* widget, gpointer data)
+{
   close_edit_window ();
   close_list_window ();
   return TRUE;
@@ -263,9 +292,10 @@ static key_binding_t *find_key_binding (const gchar *desc, const gchar *cmd,
   if (!desc && !cmd)
     return NULL;
 
-  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), iter)) {
-
-    do {
+  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), iter))
+  {
+    do
+    {
       key_binding_t *key_binding = lookup_binding (iter);
 
       /* allow either to be null, in which case we match on the other rather
@@ -275,28 +305,30 @@ static key_binding_t *find_key_binding (const gchar *desc, const gchar *cmd,
 	  (!cmd || !strcasecmp (key_binding->cmd, cmd)))
 	return key_binding;
 
-    } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (kb_store),
-				       iter));
+    } while (gtk_tree_model_iter_next (kb_model, iter));
   }
 
   return NULL;
 }
 
-static gchar* kb2str (key_binding_t *kb) {
-
-  static const gchar *text_shift = NULL, *text_control,
-		     *text_mod1, *text_mod3, *text_mod4;
+static gchar* kb2str (key_binding_t *kb)
+{
+  static struct {
+    const gchar *shift, *ctrl, *alt, *mod2, *mod3, *mod4, *mod5;
+  } text = { NULL };
   guint              l;
   gchar             *name;
   const gchar       *key;
 
-  if (!text_shift)
+  if (!text.shift)
   {
-    text_shift   = _("<Shift>");
-    text_control = _("<Control>");
-    text_mod1    = _("<Alt>");
-    text_mod3    = _("<Mod3>");
-    text_mod4    = _("<Mod4>");
+    text.shift = _("<Shift>");
+    text.ctrl  = _("<Control>");
+    text.alt   = _("<Alt>");
+    text.mod2  = _("<Mod2>");
+    text.mod3  = _("<Mod3>");
+    text.mod4  = _("<Mod4>");
+    text.mod5  = _("<Mod5>");
   }
 
   /*
@@ -305,15 +337,19 @@ static gchar* kb2str (key_binding_t *kb) {
 
   l = 1;
   if (kb->state & GDK_SHIFT_MASK)
-    l += strlen (text_shift);
+    l += strlen (text.shift);
   if (kb->state & GDK_CONTROL_MASK)
-    l += strlen (text_control);
+    l += strlen (text.ctrl);
   if (kb->state & GDK_MOD1_MASK)
-    l += strlen (text_mod1);
+    l += strlen (text.alt);
+  if (kb->state & GDK_MOD2_MASK)
+    l += strlen (text.mod2);
   if (kb->state & GDK_MOD3_MASK)
-    l += strlen (text_mod3);
+    l += strlen (text.mod3);
   if (kb->state & GDK_MOD4_MASK)
-    l += strlen (text_mod4);
+    l += strlen (text.mod4);
+  if (kb->state & GDK_MOD5_MASK)
+    l += strlen (text.mod5);
 
   if (kb->keyval == GDK_VoidSymbol)
     key = _("undef"); /* undefined */
@@ -325,15 +361,19 @@ static gchar* kb2str (key_binding_t *kb) {
   *name = 0;
 
   if (kb->state & GDK_SHIFT_MASK)
-    strcat (name, text_shift);
+    strcat (name, text.shift);
   if (kb->state & GDK_CONTROL_MASK)
-    strcat (name, text_control);
+    strcat (name, text.ctrl);
   if (kb->state & GDK_MOD1_MASK)
-    strcat (name, text_mod1);
+    strcat (name, text.alt);
+  if (kb->state & GDK_MOD2_MASK)
+    strcat (name, text.mod2);
   if (kb->state & GDK_MOD3_MASK)
-    strcat (name, text_mod3);
+    strcat (name, text.mod3);
   if (kb->state & GDK_MOD4_MASK)
-    strcat (name, text_mod4);
+    strcat (name, text.mod4);
+  if (kb->state & GDK_MOD5_MASK)
+    strcat (name, text.mod5);
 
   strcat (name, key);
 
@@ -343,9 +383,7 @@ static gchar* kb2str (key_binding_t *kb) {
 static void modify_key_binding (GtkTreeIter *iter, key_binding_t *key_binding,
 				const key_binding_t *new_binding)
 {
-  free ((char *)key_binding->keyname);
-  free ((char *)key_binding->cmd);
-  free ((char *)key_binding->desc);
+  key_binding_t old_binding = *key_binding;
 
   if (new_binding->keyval != GDK_VoidSymbol)
   {
@@ -355,6 +393,7 @@ static void modify_key_binding (GtkTreeIter *iter, key_binding_t *key_binding,
 
   key_binding->keyname = kb2str (key_binding);
   key_binding->cmd     = strdup (new_binding->cmd);
+
   if (!new_binding->desc)
   {
     int i = -1;
@@ -366,8 +405,19 @@ static void modify_key_binding (GtkTreeIter *iter, key_binding_t *key_binding,
   else
     key_binding->desc  = strdup (new_binding->desc);
 
-  gtk_list_store_set (kb_store, iter, 0, key_binding->desc, 1,
-		      key_binding->keyname, 2, key_binding, -1);
+  key_binding->std = new_binding->std &
+		     (!old_binding.desc ||
+		      !strcmp (old_binding.desc, key_binding->desc));
+
+  free ((char *)old_binding.keyname);
+  free ((char *)old_binding.cmd);
+  free ((char *)old_binding.desc);
+
+  gtk_list_store_set (kb_store, iter,
+		      COLUMN_DESC, getdesc (key_binding),
+		      COLUMN_KEY, key_binding->keyname,
+		      COLUMN_DATA, key_binding,
+		      -1);
 }
 
 
@@ -380,8 +430,8 @@ static void set_key_binding (const key_binding_t *new_binding)
 		? find_key_binding (new_binding->desc, NULL, &iter)
 		: find_key_binding (NULL, new_binding->cmd, &iter);
 
-  if (!key_binding) {
-
+  if (!key_binding)
+  {
     /* add new key binding */
 
     key_binding = g_malloc0 (sizeof (key_binding_t));
@@ -411,7 +461,7 @@ static void load_default_kb (void)
   while (default_bindings[++i].desc)
   {
     key_binding_t binding = default_bindings[i];
-    binding.desc = gettext (binding.desc);
+    binding.std = TRUE;
     set_key_binding (&binding);
   }
 }
@@ -419,47 +469,39 @@ static void load_default_kb (void)
 
 void save_key_bindings (void)
 {
-  gchar *fname = g_strconcat (g_get_home_dir(), "/.gxine/keybindings", NULL);
-  FILE  *f = fopen (fname, "w");
-  
-  if (f) {
+  gchar *fname = get_config_filename (FILE_KEYBINDINGS);
+  FILE *f = open_write (fname, _("Failed to save key bindings"));
+
+  if (f)
+  {
     GtkTreeIter iter;
 
-    fprintf (f, "<GXINEKB VERSION=\"3\">\n");
+    fprintf (f, "<GXINEKB VERSION=\"4\">\n");
 
-    if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), &iter)) {
-
-      do {
+    if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), &iter))
+    {
+      do
+      {
 	key_binding_t *key_binding = lookup_binding (&iter);
 	char *desc = xml_escape_string (key_binding->desc, XML_ESCAPE_NO_QUOTE);
 	char *cmd = xml_escape_string (key_binding->cmd, XML_ESCAPE_NO_QUOTE);
 	fprintf (f, "  <KEYBINDING>\n"
-		    "    <DESCRIPTION>%s</DESCRIPTION>\n"
+		    "    <DESCRIPTION%s>%s</DESCRIPTION>\n"
 		    "    <COMMAND>%s</COMMAND>\n"
 		    "    <KEYVAL>%d</KEYVAL>\n"
 		    "    <STATE>%d</STATE>\n"
 		    "  </KEYBINDING>\n",
-		    desc, cmd, key_binding->keyval, key_binding->state);
+		    key_binding->std ? "" : " user='yes'", desc, cmd,
+		    key_binding->keyval, key_binding->state);
 	free (desc);
 	free (cmd);
-      } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (kb_store),
-					 &iter));
+      } while (gtk_tree_model_iter_next (kb_model, &iter));
     }
 
     fprintf (f, "</GXINEKB>\n");
 
-    if (ferror (f))
-      display_error_modal (FROM_GXINE, _("Failed to save key bindings"),
-			   _("Error while writing to '%s': %s"),
-			   fname, strerror (errno));
-    if (fclose (f))
-      display_error_modal (FROM_GXINE, _("Failed to save key bindings"),
-			   _("Error when closing '%s': %s"),
-			   fname, strerror (errno));
-  } else 
-    display_error_modal (FROM_GXINE, _("Failed to save key bindings"),
-                         _("Can't open file '%s': %s"),
-			 fname, strerror (errno));
+    close_write (fname, f, _("Failed to save key bindings"));
+  }
 
   g_free (fname);
 }
@@ -532,16 +574,52 @@ static const kb_upgrade_t vdr_upgrade_to_0_5_0[] = {
   { NULL }
 };
 
+/* For upgrading from < 0.5.8 */
+static inline const char *
+xml2kb_to_std (const char *text)
+{
+  int i;
+  for (i = 0; default_bindings[i].desc; ++i)
+    if (!strcmp (text, gettext (default_bindings[i].desc)))
+      return default_bindings[i].desc;
+  return NULL;
+}
+
+/* For checking whether a description is standard (and translatable) */
+static inline gboolean
+xml2kb_is_std (const char *text)
+{
+  int i;
+  for (i = 0; default_bindings[i].desc; ++i)
+    if (!strcmp (text, default_bindings[i].desc))
+      return TRUE;
+  return FALSE;
+}
+
 static void xml2kb (xml_node_t *node, int version)
 {
   key_binding_t binding = { 0 };
-  int i;
 
   foreach_glist (node, node)
     if (!strcasecmp (node->name, "command"))
       binding.cmd = node->data;
     else if (!strcasecmp (node->name, "description"))
-      binding.desc = node->data;
+    {
+      if (version >= 4)
+      {
+	binding.std = !xml_parser_get_property (node, "user")
+		      && xml2kb_is_std (node->data);
+	binding.desc = node->data;
+      }
+      else
+      {
+	binding.desc = xml2kb_to_std (node->data);
+	binding.std = !!binding.desc;
+	if (!binding.desc)
+	  binding.desc = node->data;
+      }
+      binding.desc = strdup (binding.desc);
+    }
     else if (!strcasecmp (node->name, "keyval"))
       binding.keyval = atoi (node->data);
     else if (!strcasecmp (node->name, "state"))
@@ -555,16 +633,6 @@ static void xml2kb (xml_node_t *node, int version)
   if (version < 3)
     binding.cmd = kb_upgrade (binding.cmd, vdr_upgrade_to_0_5_0);
 
-  /* translate from English if a new translation is present and in use */
-  if (binding.desc)
-    for (i = 0; default_bindings[i].desc; ++i)
-      if (!strcmp (binding.desc, default_bindings[i].desc) &&
-	!strcmp (binding.cmd, default_bindings[i].cmd))
-      {
-	binding.desc = gettext (binding.desc);
-	break;
-      }
-
   logprintf ("key_events: desc='%s', cmd='%s', keyval=%d, state=%d\n",
 	     binding.desc ? binding.desc : "", binding.cmd, binding.keyval,
 	     binding.state);
@@ -574,21 +642,21 @@ static void xml2kb (xml_node_t *node, int version)
 
 static void load_key_bindings (void)
 {
-  gchar *fname = g_strconcat (g_get_home_dir(), "/.gxine/keybindings", NULL);
+  gchar *fname = get_config_filename (FILE_KEYBINDINGS);
   gchar *kbfile = read_entire_file (fname, NULL);
   int version = 1;
 
   if (kbfile)
   {
-    xml_node_t *node;
+    xml_node_t *root = NULL;
     xml_parser_init (kbfile, strlen (kbfile), XML_PARSER_CASE_INSENSITIVE);
 
-    if (xml_parser_build_tree (&node) >= 0)
+    if (xml_parser_build_tree (&root) >= 0)
     {
-      if (!strcasecmp (node->name, "gxinekb"))
+      if (!strcasecmp (root->name, "gxinekb"))
       {
 	xml_property_t *prop;
-	foreach_glist (prop, node->props)
+	foreach_glist (prop, root->props)
 	  if (!strcasecmp (prop->name, "version") && prop->value)
 	  {
 	    version = atoi (prop->value);
@@ -611,24 +679,33 @@ static void load_key_bindings (void)
         else if (version < 3)
           display_info (FROM_GXINE, _("Upgrading your key bindings"), "");
 
-	foreach_glist (node, node->child)
+	xml_node_t *node;
+	foreach_glist (node, root->child)
 	  if (!strcasecmp (node->name, "KEYBINDING"))
 	    xml2kb (node->child, version);
-      } else {
-	fprintf (stderr, _("key_events: error: %s is not a valid gxine keybindings file\n"),
+      }
+      else
+      {
+	g_printerr (_("key_events: error: %s is not a valid gxine keybindings file\n"),
 		 fname);
 	load_default_kb();
       }
-    } else {
-      fprintf (stderr, _("key_events: error: cannot load keybindings file %s (xml parsing failed)\n"),
+      xml_parser_free_tree (root);
+    }
+    else
+    {
+      g_printerr (_("key_events: error: cannot load keybindings file %s (XML parsing failed)\n"),
 	       fname);
       load_default_kb();
     }
 
     free (kbfile);
-  } else {
+  }
+  else
+  {
     if (errno != ENOENT)
-      fprintf (stderr, _("key_events: error: cannot open keybindings file %s\n"),
+      /* the file exists but we can't open it; permissions problem? */
+      g_printerr (_("key_events: error: cannot open keybindings file %s\n"),
 	       fname);
     load_default_kb();
   }
@@ -636,12 +713,15 @@ static void load_key_bindings (void)
   g_free(fname);
 }
 
-void kb_edit_show (void) {
-
-  if (list_visible) {
+void kb_edit_show (void)
+{
+  if (list_visible)
+  {
     close_edit_window ();
     close_list_window ();
-  } else {
+  }
+  else
+  {
     list_visible = TRUE;
     window_show (kb_bindings_list_dlg, NULL);
     gtk_tree_view_columns_autosize (GTK_TREE_VIEW(tree_view));
@@ -652,13 +732,14 @@ void kb_edit_show (void) {
 static void do_edit_binding (void)
 {
   editkey = *lookup_binding (&catch_key_iter);
-  char *title;
+  editdesc = editkey.desc;
+  editkey.desc = getdesc (&editkey);
 
   gtk_entry_set_text (GTK_ENTRY (kb_binding_desc), editkey.desc);
   gtk_text_buffer_set_text (kb_binding_command_buf, editkey.cmd, -1);
   gtk_entry_set_text (GTK_ENTRY (kb_binding_key), editkey.keyname);
 
-  title = g_strdup_printf (_("Keybinding: %s"), editkey.desc);
+  char *title = g_strdup_printf (_("Keybinding: %s"), editkey.desc);
   gtk_window_set_title (GTK_WINDOW (kb_binding_edit_dlg), title);
   free (title);
 
@@ -669,7 +750,7 @@ static void do_edit_binding (void)
 
 static void kb_new_binding (GtkWidget *widget, gpointer data)
 {
-  
+
   set_key_binding (&default_binding);
   find_key_binding (default_binding.desc, default_binding.cmd, &catch_key_iter);
   do_edit_binding ();
@@ -684,8 +765,7 @@ static void kb_delete_binding (GtkWidget *widget, gpointer data)
 
   if (gtk_tree_selection_get_selected (sel, NULL, &iter))
   {
-    GtkTreePath *path =
-      gtk_tree_model_get_path (GTK_TREE_MODEL (kb_store), &iter);
+    GtkTreePath *path = gtk_tree_model_get_path (kb_model, &iter);
     if (path)
     {
       gtk_tree_view_set_cursor (GTK_TREE_VIEW (tree_view), path, NULL, FALSE);
@@ -826,7 +906,10 @@ static gboolean do_edit_key_cb (GtkWidget *widget, GdkEventKey *event,
 
   set_binding_quote (FALSE);
 
-  editkey.keyval = gdk_keyval_to_lower (event->keyval);
+  if (event->keyval)
+    editkey.keyval = gdk_keyval_to_lower (event->keyval);
+  else
+    editkey.keyval = ~event->hardware_keycode;
   editkey.state = event->state & GXINE_MODIFIER_MASK;
 
   name = kb2str (&editkey);
@@ -869,7 +952,11 @@ static void apply_edit (void)
 
   key_binding = lookup_binding (&catch_key_iter);
 
-  editkey.desc = gtk_entry_get_text (GTK_ENTRY (kb_binding_desc));
+  const char *desc = gtk_entry_get_text (GTK_ENTRY (kb_binding_desc));
+  /* use the original English version if we have an unmodified translation */
+  editkey.desc = (editkey.std && !strcmp (desc, editkey.desc))
+		 ? editdesc : desc;
+
   gtk_text_buffer_get_bounds (kb_binding_command_buf, &start, &end);
   editkey.cmd = gtk_text_buffer_get_text (kb_binding_command_buf, &start, &end,
 					  FALSE);
@@ -892,7 +979,7 @@ static JSBool js_xine_event (int type
 #ifdef LOG
 			    , const char *log
 #endif
-			    )
+			   )
 {
   xine_event_t event;
   se_log_fncall (log);
@@ -921,11 +1008,11 @@ JS_XINE_EVENT (input_right, INPUT_RIGHT);
 JS_XINE_EVENT (input_select, INPUT_SELECT);
 
 static JSBool js_input_menu (JSContext *cx, JSObject *obj, uintN argc,
-			     jsval *argv, jsval *rval) {
-
+			     jsval *argv, jsval *rval)
+{
   se_t *se = (se_t *) JS_GetContextPrivate(cx);
   xine_event_t event;
-  int menu;
+  int32 menu;
 
   se_log_fncall ("input_menu");
 
@@ -934,7 +1021,8 @@ static JSBool js_input_menu (JSContext *cx, JSObject *obj, uintN argc,
 
   JS_ValueToInt32 (cx, argv[0], &menu);
 
-  if (menu < 1 || menu > 7) {
+  if (menu < 1 || menu > 7)
+  {
     se->print_cb (se->print_cb_data,
 		  _("error: input_menu() expects an int between 1 and 7.\n"));
     return JS_TRUE;
@@ -957,7 +1045,7 @@ JS_XINE_EVENT (input_menu3, INPUT_MENU3);
 JS_XINE_EVENT (input_previous, INPUT_PREVIOUS);
 JS_XINE_EVENT (input_next, INPUT_NEXT);
 
-static JSBool js_keybindings_show (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_keybindings_show (JSContext *cx, JSObject *obj, uintN argc,
 				   jsval *argv, jsval *rval)
 {
   se_log_fncall_checkinit ("keybindings_show");
@@ -965,7 +1053,7 @@ static JSBool js_keybindings_show (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-static JSBool js_keypad_show (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_keypad_show (JSContext *cx, JSObject *obj, uintN argc,
 			      jsval *argv, jsval *rval)
 {
   se_log_fncall_checkinit ("keypad_show");
@@ -1084,6 +1172,7 @@ static const kb_xine_event_map_t xine_vdr = {
     { "FASTFWD", XINE_EVENT_VDR_FASTFWD },
     { "FASTREW", XINE_EVENT_VDR_FASTREW },
     { "GREEN", XINE_EVENT_VDR_GREEN },
+    { "INFO", XINE_EVENT_VDR_MUTE + 2 /* ..._VDR_INFO */ },
     { "MUTE", XINE_EVENT_VDR_MUTE },
     { "PAUSE", XINE_EVENT_VDR_PAUSE },
     { "PLAY", XINE_EVENT_VDR_PLAY },
@@ -1112,7 +1201,8 @@ static const kb_xine_event_map_t xine_vdr = {
 };
 
 JSBool js_event_generic (JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
-			 jsval *rval, const char *func, const char *prefix, ...)
+			 jsval *rval, const char *func, const char *const *prefix,
+			 ...)
 {
   se_t *se = (se_t *) JS_GetContextPrivate(cx);
   xine_event_t event;
@@ -1143,12 +1233,22 @@ JSBool js_event_generic (JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
   if (prefix)
   {
     const char *mrl = player_get_cur_mrl ();
-    if (mrl && strncasecmp (mrl, prefix, strlen (prefix)))
-      return JS_TRUE;
+    if (mrl)
+    {
+      while (*prefix)
+      {
+	if (!strncasecmp (mrl, *prefix, strlen (*prefix)))
+	  break;
+	++prefix;
+      }
+      if (!*prefix) /* no match found */
+        return JS_TRUE;
+    }
   }
 
   str = JS_ValueToString (cx, argv[0]);
   evstr = JS_GetStringBytes (str);
+  event.type = 0;
 
   va_start (ap, prefix);
   while ((evlist = va_arg (ap, const kb_xine_event_map_t *)))
@@ -1159,7 +1259,7 @@ JSBool js_event_generic (JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
     event.type = kb_xine_event_lookup (NULL, evstr);
 
   if (!event.type)
-    return JS_FALSE;
+    return JS_TRUE;
 
   if (event.type != -1)
   {
@@ -1182,7 +1282,8 @@ static JSBool js_event (JSContext *cx, JSObject *obj, uintN argc,
 static JSBool js_vdr (JSContext *cx, JSObject *obj, uintN argc,
 		      jsval *argv, jsval *rval)
 {
-  return js_event_generic (cx, obj, argc, argv, rval, "vdr", "vdr:/",
+  const char *const prefixes[] = { "vdr:/", "vdr-socket:/", NULL };
+  return js_event_generic (cx, obj, argc, argv, rval, "vdr", prefixes,
 			   &xine_vdr, NULL);
 }
 
@@ -1213,19 +1314,19 @@ static void edit_response_cb (GtkDialog *dbox, int response, gpointer data)
   }
 }
 
-void key_events_init (void) {
-
+void key_events_init (void)
+{
   GtkWidget            *b, *scrolled_window, *hbox;
   GtkCellRenderer      *cell;
   GtkTreeViewColumn    *column;
-   
+
   /*
    * init list store
    */
 
   kb_store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
+  kb_model = GTK_TREE_MODEL (kb_store);
 
-  default_binding.desc = gettext (default_binding.desc);
   load_key_bindings ();
 
   /*
@@ -1245,19 +1346,19 @@ void key_events_init (void) {
 
   /* add a nice tree view widget here */
 
-  tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(kb_store));  
+  tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(kb_store));
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree_view), TRUE);
   gtk_tree_view_set_reorderable (GTK_TREE_VIEW (tree_view), TRUE);
 
   cell = gtk_cell_renderer_text_new ();
   column = gtk_tree_view_column_new_with_attributes (_("Action"), cell,
-						     "text", 0, NULL);
+						     "text", COLUMN_DESC, NULL);
   gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN(column), TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view),
 			       GTK_TREE_VIEW_COLUMN (column));
 
   column = gtk_tree_view_column_new_with_attributes (_("Accelerator key"), cell,
-						     "text", 1, NULL);
+						     "text", COLUMN_KEY, NULL);
   gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN(column),
 				   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view),
@@ -1335,17 +1436,15 @@ void key_events_init (void) {
 				   GTK_RESPONSE_OK);
   gtk_window_set_default_size (GTK_WINDOW (kb_binding_edit_dlg), 400, 200);
 
-  kb_binding_edit_accel = gtk_accel_group_new ();
-  gtk_window_add_accel_group (GTK_WINDOW (kb_binding_edit_dlg),
-			      kb_binding_edit_accel);
+  ui_add_undo_response (kb_binding_edit_dlg, NULL);
 
   b = gtk_table_new (3, 3, FALSE);
 
   kb_binding_desc = gtk_entry_new ();
   gtk_entry_set_max_length (GTK_ENTRY(kb_binding_desc), 32);
-  add_table_row_items (b, 0, FALSE, _("Description"), 1, kb_binding_desc);
+  add_table_row_items (b, 0, FALSE, _("Description"), kb_binding_desc, NULL);
   gtk_entry_set_activates_default (GTK_ENTRY(kb_binding_desc), TRUE);
-  
+
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -1357,16 +1456,16 @@ void key_events_init (void) {
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(kb_binding_command),
 			       GTK_WRAP_NONE);
   gtk_container_add (GTK_CONTAINER (scrolled_window), kb_binding_command);
-  add_table_row_items (b, 1, TRUE, _("Commands"), 1, scrolled_window);
+  add_table_row_items (b, 1, TRUE, _("Commands"), scrolled_window, NULL);
 
   kb_binding_key = gtk_entry_new ();
   gtk_editable_set_editable (GTK_EDITABLE(kb_binding_key), FALSE);
   gtk_entry_set_activates_default (GTK_ENTRY(kb_binding_key), TRUE);
   kb_binding_quote = gtk_toggle_button_new_with_mnemonic (_("_Grab"));
-  add_table_row_items (b, 2, FALSE, _("Accelerator key"), 2, kb_binding_key,
-		       kb_binding_quote);
+  add_table_row_items (b, 2, FALSE, _("Accelerator key"),
+		       kb_binding_key, kb_binding_quote, NULL);
 
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (kb_binding_edit_dlg)->vbox ), b);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (kb_binding_edit_dlg)->vbox) , b);
 
   g_object_connect (G_OBJECT (kb_binding_edit_dlg),
 	"signal::delete-event", G_CALLBACK (close_edit_cb), NULL,
@@ -1410,5 +1509,7 @@ void key_events_init (void) {
     se_defuns (gse, NULL, defs);
   }
 
-  keypad = widget_create_from_xml ("keypad.xml", "gxine-keypad", NULL, TRUE);
+  keypad = widget_create_from_xml ("keypad.xml", "gxine-keypad", NULL, TRUE, FALSE);
+  if (!keypad)
+    gtk_action_set_sensitive (action_items.keypad, FALSE);
 }
