@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2005 the xine-project
+ * Copyright (C) 2001-2006 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -13,10 +13,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: main.c,v 1.149 2005/11/10 18:42:15 dsalt Exp $
+ * $Id: main.c,v 1.163 2006/04/08 21:34:50 dsalt Exp $
  *
  * gtk2 ui for xine
  *
@@ -26,6 +26,7 @@
 #include "globals.h"
 
 #include <string.h>
+#include <signal.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -34,7 +35,6 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <pthread.h>
 
 #ifdef HAVE_GETOPT_LONG
 #include <getopt.h>
@@ -45,6 +45,7 @@
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib.h>
+#include <glib/gmessages.h>
 
 #include "engine.h"
 #include "player.h"
@@ -69,6 +70,7 @@
 #include "utils.h"
 #include "vis.h"
 #include "post.h"
+#include "console_output.h"
 
 /*
  * globals
@@ -76,10 +78,9 @@
 
 int             verbosity;
 GtkWidget      *app = NULL;
-gchar          *plugindir, *bindir, *logodir, *pixmapdir, *icondir, *miscdir;
+gchar          *plugindir, *bindir, *logodir, *pixmapdir, *icondir, *miscdir, *confdir;
 gchar          *video_driver_id=NULL;
 gchar          *audio_driver_id=NULL;
-GAsyncQueue    *js_queue;
 
 static void
 gxine_try_remote (int argc, char *argv[], gboolean enqueue, gboolean autoplay)
@@ -94,26 +95,18 @@ gxine_try_remote (int argc, char *argv[], gboolean enqueue, gboolean autoplay)
   if (!enqueue)
     server_client_send ("playlist_clear();\n");
 
-  if (optind<argc) {
+  if (optind < argc)
+  {
     int i;
-
-    for (i=optind; i<argc; i++)
+    for (i = optind; i < argc; ++i)
     {
-      char *mrl = argv[i];
-
-      if (mrl[0] != '/' && !strstr (mrl, ":/"))
-      {
-	/* make filename an absolute pathname */
-	char  buf[1024];
-	getcwd (buf, sizeof (buf));
-	mrl = g_strconcat (buf, "/", mrl, NULL);
-      }	
-
       if (autoplay && i == optind)
         server_client_send ("playlist_play (");
       server_client_send ("playlist_add (\"");
-      server_client_send (mrl);
+      char *uri = make_path_uri (argv[i]);
+      server_client_send (uri ? uri : argv[i]);
       server_client_send ((autoplay && i == optind) ? "\"));\n" : "\");\n");
+      free (uri);
     }
   }
 
@@ -129,31 +122,41 @@ static gboolean splash_destroy (gpointer splash)
 static void splash_show (void)
 {
   gchar     *pathname;
-  GtkWidget *img;
-  GtkWidget *splash;
+  GtkWidget *splash, *fixed, *img, *text;
 
-  gdk_threads_enter();  
+  gdk_threads_enter();
 
-  splash = gtk_window_new (GTK_WINDOW_POPUP);
+  splash = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  GtkWindow *w = GTK_WINDOW (splash);
+  gtk_window_set_decorated (w, FALSE);
+  gtk_window_set_type_hint (w, GDK_WINDOW_TYPE_HINT_SPLASHSCREEN);
+  gtk_window_set_position (w, GTK_WIN_POS_CENTER_ALWAYS);
+  gtk_window_set_keep_above (w, TRUE);
+  gtk_window_stick (w);
 
-  gtk_window_set_decorated (GTK_WINDOW (splash), FALSE);
-
-  gtk_window_set_type_hint (GTK_WINDOW (splash),
-			    GDK_WINDOW_TYPE_HINT_SPLASHSCREEN);
-  gtk_window_set_position (GTK_WINDOW (splash), GTK_WIN_POS_CENTER_ALWAYS);
-  gtk_window_set_keep_above (GTK_WINDOW (splash), TRUE);
-
-  pathname = g_strconcat (pixmapdir, "/splash.png", NULL);
+  pathname = g_build_filename (pixmapdir, "splash.png", NULL);
   img = gtk_image_new_from_file (pathname);
   g_free (pathname);
 
-  gtk_container_add (GTK_CONTAINER (splash),img);
+  fixed = gtk_fixed_new ();
+  gtk_container_add (GTK_CONTAINER (splash), fixed);
+  gtk_fixed_put (GTK_FIXED (fixed), img, 0, 0);
+
+  text = gtk_label_new ("<span font_desc='sans 12' color='#DDDDDD'>"VERSION"</span>");
+  gtk_label_set_use_markup (GTK_LABEL (text), TRUE);
+  gtk_misc_set_alignment (GTK_MISC (text), 1, 1);
+  gtk_fixed_put (GTK_FIXED (fixed), text, 0, 0);
 
   gtk_widget_show_all (splash);
+  /* the widgets need to be realised before we can move the label */
+  gtk_fixed_move (GTK_FIXED (fixed), text,
+		  img->allocation.width - text->allocation.width,
+		  img->allocation.height - text->allocation.height);
+
   gtk_timeout_add (4000, splash_destroy, splash);
 
   do_pending_events ();
-  gdk_threads_leave();  
+  gdk_threads_leave();
 }
 
 #ifndef GLIB_USES_SYSTEM_MALLOC
@@ -179,26 +182,25 @@ gpointer xrealloc (gpointer p, gsize size)
 static gboolean post_init_configure (void)
 {
   xine_cfg_entry_t entry;
-  GtkVideo *v = GTK_VIDEO(gtv);
 
   if (xine_config_lookup_entry (xine, "gui.post_plugins.deinterlace", &entry))
-    gtk_video_set_post_plugins_deinterlace (v, entry.str_value);
+    gtk_video_set_post_plugins_deinterlace ((GtkVideo *)gtv, entry.str_value);
 
   if (xine_config_lookup_entry (xine, "gui.post_plugins.video", &entry))
-    gtk_video_set_post_plugins_video (v, entry.str_value);
+    gtk_video_set_post_plugins_video ((GtkVideo *)gtv, entry.str_value);
 
   if (xine_config_lookup_entry (xine, "gui.post_plugins.audio", &entry))
-    gtk_video_set_post_plugins_audio (v, entry.str_value, audio_port);
+    gtk_video_set_post_plugins_audio ((GtkVideo *)gtv, entry.str_value, audio_port);
 
   if (xine_config_lookup_entry (xine, "gui.post_plugins.deinterlace_enable",
 				&entry))
-    gtk_video_set_use_post_plugins_deinterlace (v, entry.num_value);
+    gtk_video_set_use_post_plugins_deinterlace ((GtkVideo *)gtv, entry.num_value);
 
   if (xine_config_lookup_entry (xine, "gui.post_plugins.video_enable", &entry))
-    gtk_video_set_use_post_plugins_video (v, entry.num_value);
+    gtk_video_set_use_post_plugins_video ((GtkVideo *)gtv, entry.num_value);
 
   if (xine_config_lookup_entry (xine, "gui.post_plugins.audio_enable", &entry))
-    gtk_video_set_use_post_plugins_audio (v, entry.num_value, audio_port);
+    gtk_video_set_use_post_plugins_audio ((GtkVideo *)gtv, entry.num_value, audio_port);
 
   if (xine_config_lookup_entry (xine, "gui.post_plugins.audio_visualisation",
 				&entry))
@@ -219,11 +221,32 @@ static gboolean set_fullscreen (gpointer data)
   return FALSE;
 }
 
+#ifdef WITH_WATCHDOG
+/* Watchdog functions */
+
+static gboolean watchdog_timer_reset (gpointer data)
+{
+  alarm (30);
+  return TRUE;
+}
+
+static void watchdog_timeout (int sig)
+{
+  /* if this happens, we have a problem... */
+  g_printerr (_("gxine: killed by watchdog bite\n"));
+  abort ();
+}
+#endif
+
+/* Main */
+
 int main(int argc, char* argv[])
 {
   gboolean enqueue, autoplay, fullscreen;
+  gboolean no_connect;
   int optstate;
-  gboolean show_splash = TRUE;
+  int show_splash = 2; /* default */
+  const char *cmd = NULL;
 
 #ifndef GLIB_USES_SYSTEM_MALLOC
   GMemVTable vtab = { xmalloc, xrealloc, free, NULL, NULL, NULL };
@@ -244,6 +267,7 @@ int main(int argc, char* argv[])
   pixmapdir = getenv("GXINE_PIXMAPDIR")	? : g_strconcat(GXINE_PIXMAPDIR, NULL);
   icondir   = getenv("GXINE_ICONDIR")	? : g_strconcat(GXINE_ICONDIR, NULL);
   miscdir   = getenv("GXINE_MISCDIR")	? : g_strconcat(GXINE_MISCDIR, NULL);
+  confdir   = getenv("GXINE_CONFDIR")	? : g_strconcat(GXINE_CONFDIR, NULL);
 
   /* set up our gtkrc stuff */
   {
@@ -264,9 +288,9 @@ int main(int argc, char* argv[])
 	break;
     /* insert our system gtkrc filename before it (strdup for convenience) */
     memmove (copy + i + 1, copy + i, (length - i) * sizeof (gchar *));
-    copy[i] = strdup (GXINE_CONFDIR "/gtkrc");
+    copy[i] = g_build_filename (confdir, FILE_GTKRC, NULL);
     /* now append our user gtkrc, terminate the list and tell GTK+ about it */
-    copy[length + 1] = g_strconcat (home, "/.gxine/gtkrc", NULL);
+    copy[length + 1] = get_config_filename (FILE_GTKRC);
     copy[length + 2] = NULL;
     gtk_rc_set_default_files (copy);
     /* free the list */
@@ -293,9 +317,8 @@ int main(int argc, char* argv[])
 #endif
 
   g_thread_init (NULL);
+  console_output_init ();
   gdk_threads_init ();
-  gtk_init(&argc, &argv);
-  gtk_window_set_auto_startup_notification (FALSE);
 
   /*
    * parse command line arguments
@@ -305,20 +328,24 @@ int main(int argc, char* argv[])
   enqueue = FALSE;
   autoplay = TRUE;
   fullscreen = FALSE;
+  no_connect = FALSE;
   optstate = 0;
   for (;;)
   {
-#define OPTS "hvaefs:V:A:S"
+#define OPTS "hvaefV:A:sSc:n"
 #ifdef HAVE_GETOPT_LONG
     static struct option longopts[] = {
       { "help", no_argument, NULL, 'h' },
+      { "version", no_argument, NULL, 1 },
       { "verbose", no_argument, NULL, 'v' },
       { "add", no_argument, NULL, 'a' },
       { "enqueue", no_argument, NULL, 'e' },
       { "full-screen", no_argument, NULL, 'f' },
       { "video", required_argument, NULL, 'V' },
       { "audio", required_argument, NULL, 'A' },
+      { "splash", no_argument, NULL, 's' },
       { "no-splash", no_argument, NULL, 'S' },
+      { "command", required_argument, NULL, 'c' },
       { NULL }
     };
     int index = 0;
@@ -329,10 +356,14 @@ int main(int argc, char* argv[])
     if (opt == -1)
       break;
 
-    switch (opt) {
+    switch (opt)
+    {
     case 'h':
       optstate |= 1;
       break;
+    case 1:
+      optstate |= 4;
+      break;
     case 'v':
       verbosity++;
       break;
@@ -353,9 +384,18 @@ int main(int argc, char* argv[])
     case 'A':
       audio_driver_id = optarg;
       break;
+    case 's':
+      show_splash = TRUE;
+      break;
     case 'S':
       show_splash = FALSE;
       break;
+    case 'c':
+      cmd = optarg;
+      break;
+    case 'n':
+      no_connect = TRUE;
+      break;
     default:
       optstate |= 2;
       break;
@@ -363,7 +403,7 @@ int main(int argc, char* argv[])
   }
 
   if (optstate & 1)
-    printf (_("\
+    g_print (_("\
 gxine %s (%s)\n\
 usage: %s [options] [MRLs...]\n\
 options:\n\
@@ -371,26 +411,54 @@ options:\n\
   -A, --audio DRIVER	try to use this audio driver\n\
   -V, --video DRIVER	try to use this video driver\n\
   -S, --no-splash	don't show the splash window\n\
+  -c, --command TEXT	Javascript command(s) to be executed at startup\n\
   -a, --add		don't clear the playlist, play the first new item\n\
   -e, --enqueue		don't clear the playlist, don't play the new items\n\
   -f, --full-screen	start in full-screen mode\n\
   -v, --verbose		be more verbose\n\
 \n"), VERSION, VENDOR_PKG_VERSION, argv[0]);
+  else if (optstate & 4)
+    g_print ("gxine %s (%s)\n%s\n%s", VERSION, VENDOR_PKG_VERSION,
+	    _("© 2002-2006 the xine project team"), _("\
+This is free software; see the source for copying conditions.  There is NO\n\
+warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE,\n\
+to the extent permitted by law.\n"));
 
   if (optstate & 2)
   {
-    puts (_("gxine: invalid option (try -h or --help)"));
+    g_printerr (_("gxine: invalid option (try -h or --help)\n"));
     return 1;
   }
 
   if (optstate)
     return 0;
 
+  /* init gtk windowing */
+
+  gtk_init(&argc, &argv);
+  gtk_window_set_auto_startup_notification (FALSE);
+
+  /*
+   * find out if gxine is already running, if so
+   * just pass on the files to play
+   */
+
+  sched_yield ();
+  if (!no_connect)
+    gxine_try_remote (argc, argv, enqueue, autoplay);
+
+  /* start using X... */
+  if (!XInitThreads ())
+  {
+    g_printerr (_("gtkvideo: XInitThreads failed - looks like you don't have a thread-safe xlib.\n"));
+    return 2;
+  }
+
   /*
    * make sure ~/.gxine exists
    */
   {
-    char *fname = g_strconcat (g_get_home_dir(), "/.gxine", NULL);
+    char *fname = get_config_filename (NULL);
     int r = ensure_path_exists (fname, 0700);
     if (r)
       display_warning
@@ -401,44 +469,54 @@ options:\n\
     g_free (fname);
   }
 
+#ifdef WITH_WATCHDOG
   /*
-   * find out if gxine is already running, if so
-   * just pass on the files to play
+   * Watchdog
    */
 
-  gxine_try_remote (argc, argv, enqueue, autoplay);
+  g_timeout_add (2500, watchdog_timer_reset, NULL); /* <- will set the alarm */
+  signal (SIGALRM, watchdog_timeout);
+#endif
 
+  /*
+   * init xine, set up script engine
+   */
+
+  engine_init ();
   server_setup ();
 
-  /* start using X... */
-  if (!XInitThreads ())
-  {
-    fprintf (stderr, _("gtkvideo: XInitThreads failed - looks like you don't have a thread-safe xlib.\n"));
-    return 2;
-  }
-
   /*
    * a splash screen for the impatient
    */
 
+  if (show_splash == 2)
+  {
+    xine_cfg_entry_t entry;
+    show_splash = !xine_config_lookup_entry (xine, "gui.show_splash", &entry)
+		  || entry.num_value;
+  }
+
   if (show_splash)
     splash_show ();
 
-  /*
-   * init xine, set up skript engine, main window
-   */
-
   gdk_threads_enter ();
   do_pending_events ();
 
-  js_queue = g_async_queue_new ();
-  g_async_queue_ref (js_queue);
+  /*
+   * set up player and some UI basics
+   */
 
-  engine_init ();
   player_init ();
-  ui_init	   ();
+  ui_init ();
 
   gtk_window_set_default_icon_name (GXINE_LOGO);
+  
+
+  {
+    char *fname = get_config_filename (FILE_ACCELS);
+    gtk_accel_map_load (fname);
+    free (fname);
+  }
 
   do_pending_events ();
   gtk_window_set_auto_startup_notification (TRUE);
@@ -447,7 +525,7 @@ options:\n\
   do_pending_events ();
 
   /*
-   * create all dialogs (invisible)
+   * create all dialogue boxes etc.
    */
 
   file_dialog_init ();
@@ -483,21 +561,19 @@ options:\n\
    * argument parsing
    */
 
-  if (optind<argc) {
+  if (optind < argc)
+  {
     int i;
     static int first = -1;
 
     if (!enqueue)
       playlist_clear();
 
-    for (i=optind; i<argc; i++) {
-
-      if (first<0) {
+    for (i = optind; i < argc; ++i)
+      if (first < 0)
 	first = playlist_add_mrl (argv[i], -1);
-      } else {
+      else
 	playlist_add_mrl (argv[i], -1);
-      }
-    }
 
     if (first < 0)
       gtk_init_add ((GtkFunction) playlist_logo, NULL);
@@ -513,6 +589,10 @@ options:\n\
   gtk_init_add ((GtkFunction) ui_post_init, NULL);
   server_start ();
 
+  /* execute any command-line JS */
+  if (cmd)
+    se_eval (gse, cmd, NULL, NULL, NULL, "--command");
+
   //gdk_threads_enter();
   gtk_main();
   //gdk_threads_leave();
