@@ -1,4 +1,4 @@
-/* 
+/*
  * Copyright (C) 2003-2005 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
@@ -13,9 +13,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: menu.c,v 1.68 2005/11/11 04:29:16 dsalt Exp $
+ * $Id: menu.c,v 1.82 2006/04/08 21:34:50 dsalt Exp $
  *
  * main/context menu creation / handling
  */
@@ -52,61 +52,53 @@ action_items_t action_items = { NULL };
 #define get_radio(name) GTK_RADIO_ACTION(get_action ((name)))
 #define get_radio_group(name) gtk_radio_action_get_group (get_radio ((name)))
 
-/*
- * context menu
- */
-
-#if 0
-static gint context_menu_cb (GtkWidget *widget, GdkEvent *event) {
-
-  if (event->type == GDK_BUTTON_PRESS) {
-    GdkEventButton *bevent = (GdkEventButton *) event; 
-
-    if (bevent->button == 3) {
-      gtk_menu_popup (GTK_MENU(widget), NULL, NULL, NULL, NULL,
-		      bevent->button, bevent->time);
-      /* Tell calling code that we have handled this event; the buck
-       * stops here. */
-      return TRUE;
-    }
-  }
-  
-  /* Tell calling code that we have not handled this event; pass it on. */
-  return FALSE;
-}
-#endif
-
-
 static const char type[][8] = { "menubar", "popup" };
 
-const char *menu_get_tree_name (int tree)
+const char *menu_get_tree_name (unsigned int tree)
 {
-  return (tree >= 0 && tree < G_N_ELEMENTS(type)) ? type[tree] : NULL;
+  return tree < G_N_ELEMENTS(type) ? type[tree] : NULL;
 }
 
 static void autoplay_cb (GtkAction *action, const char *data)
 {
-  char **mrls;
-  int    n, i, pos;
+  int n, i, pos;
 
-  mrls = xine_get_autoplay_mrls (xine, data, &n);
-
-  if (!mrls) {
-    char *title;
-    playlist_logo();
-    title = g_markup_printf_escaped (_("Autoplay input plugin '%s' failed"),
-				     data);
+  char **mrls = xine_get_autoplay_mrls (xine, data, &n);
+  if (!mrls)
+  {
+    playlist_logo (NULL);
+    char *title = g_markup_printf_escaped (_("Autoplay input plugin ‘%s’ failed"),
+					   data);
     display_error (FROM_XINE, title, _("Check engine output for further details."));
     free (title);
     return;
   }
 
+  playlist_flush (PLAY_ITEM_AUTOPLAY);
   pos = -1;
   for (i = 0; mrls[i]; ++i)
+  {
+    /* don't duplicate the first item if it's already listed */
     if (pos < 0)
-      pos = playlist_add_mrl (mrls[i], -1);
+    {
+      int j;
+      for (j = playlist_size () - 1; j >= 0; --j)
+        if (!strcmp (playlist_get_item (j)->mrl, mrls[i]))
+          break;
+      if (j >= 0)
+      {
+        pos = j;
+        continue;
+      }
+    }
+
+    play_item_t *item = play_item_new (NULL, mrls[i], 0);
+    item->type = PLAY_ITEM_AUTOPLAY;
+    if (pos < 0)
+      pos = playlist_add (item, -1);
     else
-      playlist_add_mrl (mrls[i], -1);
+      playlist_add (item, -1);
+  }
 
   playlist_play (pos);
 }
@@ -131,7 +123,8 @@ static size_t add_autoplay_actions (GtkUIManager *ui)
     { "CD", GTK_STOCK_CDROM },
   };
 
-  if (!autoplay_ids) {
+  if (!autoplay_ids)
+  {
     logprintf ("menu: warning: didn't get any autoplay entries\n");
     return 0;
   }
@@ -143,11 +136,11 @@ static size_t add_autoplay_actions (GtkUIManager *ui)
 
   for (i = 0; autoplay_ids[i]; ++i)
   {
-    int j = sizeof (stock) / sizeof (stock[0]);
+    int j = G_N_ELEMENTS (stock);
     while (j--)
       if (!strcmp (autoplay_ids[i], stock[j].id))
 	break;
-    snprintf (name, sizeof (name), AUTOPLAY_NAME"%d", i);
+    snprintf (name, sizeof (name), AUTOPLAY_NAME"%zu", i);
     GtkAction *action = gtk_action_new (name, autoplay_ids[i], NULL,
 					j < 0 ? NULL : stock[j].stock);
     g_signal_connect (G_OBJECT(action), "activate",
@@ -188,7 +181,7 @@ static size_t add_vis_actions (GtkUIManager *ui, GtkAction *group_action)
 
   for (i = 0; pol[i]; ++i)
   {
-    snprintf (name, sizeof (name), VIS_NAME"%d", i);
+    snprintf (name, sizeof (name), VIS_NAME"%zu", i);
     GtkRadioAction *action = gtk_radio_action_new
 			       (name, pol[i], NULL, NULL, i + 1);
     gtk_radio_action_set_group (action, radiogroup);
@@ -215,7 +208,7 @@ void menu_add_items (GtkUIManager *ui, const char *const prefix,
     size_t j = count;
     while (j--)
     {
-      snprintf (name, sizeof (name), "%s%d", label, j);
+      snprintf (name, sizeof (name), "%s%zu", label, j);
       gtk_ui_manager_add_ui (ui, merge, path, name, name,
 			     GTK_UI_MANAGER_MENUITEM, TRUE);
     }
@@ -273,7 +266,7 @@ JS_CB (Quit, "exit ();")
 /* View */
 JS_CB_T (Fullscreen)
 {
-  gtk_video_set_fullscreen (GTK_VIDEO(gtv), v);
+  gtk_video_set_fullscreen ((GtkVideo *)gtv, v);
   if (v)
     window_fs_toolbar_restore ();
   else
@@ -284,9 +277,10 @@ JS_CB_R (FSToolbarPos) { window_fs_toolbar_position (v); }
 JS_CB_T (WMToolbarVisible) { window_wm_toolbar_show (v); }
 JS_CB_T (WMToolbarSnap) { window_wm_toolbar_set_snap (v); }
 JS_CB (Keypad, "keypad_show ();")
-JS_CB_R (Window) { gtk_video_rescale (GTK_VIDEO(gtv), (double) v / 1e+6); }
-JS_CB_T (WindowAutosize) { gtk_video_set_auto_resize (GTK_VIDEO(gtv), v); }
-JS_CB_T (WindowMagLow) { gtk_video_set_auto_rescale (GTK_VIDEO(gtv), v); }
+JS_CB_R (Window) { gtk_video_rescale ((GtkVideo *)gtv, (double) v / 1e+6); }
+JS_CB (WindowRevert, "set_video_size (-1);")
+JS_CB_T (WindowAutosize) { gtk_video_set_auto_resize ((GtkVideo *)gtv, v); }
+JS_CB_T (WindowMagLow) { gtk_video_set_auto_rescale ((GtkVideo *)gtv, v); }
 JS_CB (ZoomIn, "vo_zoom.v += 5;")
 JS_CB (ZoomOut, "vo_zoom.v -= 5;")
 JS_CB (Zoom100, "vo_zoom.v = 100;")
@@ -301,9 +295,9 @@ JS_CB_R (Subtitles)
 JS_CB (VideoSettings, "settings_show (0);")
 JS_CB_T (Deinterlace)
 {
-  gtk_video_set_use_post_plugins_deinterlace (GTK_VIDEO(gtv), v);
+  gtk_video_set_use_post_plugins_deinterlace ((GtkVideo *)gtv, v);
 }
-JS_CB_T (VideoPP) { gtk_video_set_use_post_plugins_video (GTK_VIDEO(gtv), v); }
+JS_CB_T (VideoPP) { gtk_video_set_use_post_plugins_video ((GtkVideo *)gtv, v); }
 JS_CB (DeinterlaceConf, "deinterlace_show ();")
 JS_CB (VideoPPConf, "postproc_video_show ();")
 
@@ -312,7 +306,7 @@ JS_CB (AudioSettings, "settings_show (1);")
 JS_CB (AudioEq, "settings_show (2);")
 JS_CB_T (AudioPP)
 {
-  gtk_video_set_use_post_plugins_audio (GTK_VIDEO(gtv), v, audio_port);
+  gtk_video_set_use_post_plugins_audio ((GtkVideo *)gtv, v, audio_port);
 }
 JS_CB (AudioPPConf, "postproc_audio_show ();")
 
@@ -323,10 +317,15 @@ JS_CB (MediaImport, "import_mediamarks ();")
 
 /* Help */
 JS_CB (About, "about_show ();")
+
+/* Configure */
 JS_CB (EngineLog, "log_show ();")
 JS_CB (JSConsole, "js_console_show ();")
 JS_CB (StreamInfo, "stream_info_show ();")
 JS_CB (SetupWizards, "run_wizards ();")
+#ifdef HAVE_LIRC
+JS_CB (ReloadLIRC, "reload_lirc ();")
+#endif
 
 #define CB(name, stock, label, key) \
   { #name, (stock), (label), (key), NULL, G_CALLBACK(js_##name##_cb) }
@@ -335,16 +334,20 @@ static const GtkActionEntry main_actions[] = {
   { "FileMenu",		NULL,			N_("_File") },
   CB(Open,		GTK_STOCK_OPEN,		N_("_Open..."),			"<control>O"),
   CB(MRL,		GTK_STOCK_OPEN,		N_("Open _MRL..."),		"<control>M"),
-  CB(Playlist,		GXINE_MEDIA_PLAYLIST,	N_("Play_list..."),		NULL),
+  CB(Playlist,		GTK_STOCK_INDEX,	N_("Play_list..."),		NULL),
   { "ConfigMenu",	GTK_STOCK_PREFERENCES,	N_("_Configure") },
   CB(Prefs,		NULL,			N_("_Preferences..."),		NULL),
   CB(Keys,		NULL,			N_("_Keybindings..."),		NULL),
   CB(Startup,		NULL,			N_("_Startup script..."),	NULL),
+#ifdef HAVE_LIRC
+  CB(ReloadLIRC,	GTK_STOCK_REFRESH,	N_("_Reload LIRC"),		NULL),
+#endif
   CB(Snapshot,		NULL,			N_("_Snapshot..."),		"<control>S"),
   CB(Quit,		GTK_STOCK_QUIT,		N_("_Quit"),			"<control>Q"),
   { "ViewMenu",		NULL,			N_("_View") },
   { "ToolbarMenu",	NULL,			N_("_Toolbars") },
   { "WindowMenu",	NULL,			N_("_Window size") },
+  CB(WindowRevert,	NULL,			N_("Re_set"),			NULL),
   CB(Keypad,		NULL,			N_("_Keypad..."),		NULL),
   CB(ZoomIn,		GTK_STOCK_ZOOM_IN,	N_("Zoom _in"),			NULL),
   CB(ZoomOut,		GTK_STOCK_ZOOM_OUT,	N_("Zoom _out"),		NULL),
@@ -353,12 +356,12 @@ static const GtkActionEntry main_actions[] = {
   { "VisMenu",		NULL,			N_("_Visualisations"),		NULL },
   { "SubsMenu",		NULL,			N_("_Subtitles"),		NULL },
   { "VideoMenu",	NULL,			N_("V_ideo"),			NULL },
-  CB(VideoSettings,	GTK_STOCK_PROPERTIES,	N_("_Settings..."),		NULL),
+  CB(VideoSettings,	GTK_STOCK_PROPERTIES,	/* Video */ N_("_Settings..."),	NULL),
   { "VideoConfigMenu",	NULL,			N_("_Configure"),		NULL },
   CB(DeinterlaceConf,	GTK_STOCK_PREFERENCES,	N_("_Deinterlace..."),		NULL),
   CB(VideoPPConf,	GTK_STOCK_PREFERENCES,	N_("_Post-processing..."), 	NULL),
   { "AudioMenu",	NULL,			N_("_Audio"),			NULL },
-  CB(AudioSettings,	GTK_STOCK_PROPERTIES,	N_("_Settings..."),		NULL),
+  CB(AudioSettings,	GTK_STOCK_PROPERTIES,	/* Audio */ N_("_Settings..."),	NULL),
   CB(AudioEq,		GTK_STOCK_PROPERTIES,	N_("_Equaliser..."),		NULL),
   { "AudioConfigMenu",	NULL,			N_("_Configure"),		NULL },
   CB(AudioPPConf,	GTK_STOCK_PREFERENCES,	N_("_Post-processing..."),	NULL),
@@ -415,7 +418,9 @@ fstoolbar_actions[] = {
 window_actions[] = {
   { "WindowOther",	NULL, "",		NULL, NULL, 0 },
   { "Window50",		NULL, N_("_50%"),	NULL, NULL, 50e+6 },
+  { "Window75",		NULL, N_("_75%"),	NULL, NULL, 75e+6 },
   { "Window100",	NULL, N_("_100%"),	NULL, NULL, 100e+6 },
+  { "Window150",	NULL, N_("15_0%"),	NULL, NULL, 150e+6 },
   { "Window200",	NULL, N_("_200%"),	NULL, NULL, 200e+6 },
   { "WindowDynamic",	NULL, "",		NULL, NULL, -1 },
 },
@@ -443,6 +448,14 @@ subtitle_actions[] = {
   { "Subtitles5",	NULL, N_("Channel _5"),	NULL, NULL, 5 },
   { "Subtitles6",	NULL, N_("Channel _6"),	NULL, NULL, 6 },
   { "Subtitles7",	NULL, N_("Channel _7"),	NULL, NULL, 7 },
+  { "Subtitles8",	NULL, N_("Channel _8"),	NULL, NULL, 8 },
+  { "Subtitles9",	NULL, N_("Channel _9"),	NULL, NULL, 9 },
+  { "Subtitles10",	NULL, N_("Channel 10"),	NULL, NULL, 10 },
+  { "Subtitles11",	NULL, N_("Channel 11"),	NULL, NULL, 11 },
+  { "Subtitles12",	NULL, N_("Channel 12"),	NULL, NULL, 12 },
+  { "Subtitles13",	NULL, N_("Channel 13"),	NULL, NULL, 13 },
+  { "Subtitles14",	NULL, N_("Channel 14"),	NULL, NULL, 14 },
+  { "Subtitles15",	NULL, N_("Channel 15"),	NULL, NULL, 15 },
 };
 
 static GtkWidget *add_menu_ui (GtkUIManager *ui, int i)
@@ -457,6 +470,9 @@ static GtkWidget *add_menu_ui (GtkUIManager *ui, int i)
 	   "<menuitem action='Prefs' />\n"
 	   "<menuitem action='Keys' />\n"
 	   "<menuitem action='Startup' />\n"
+#ifdef HAVE_LIRC
+	   "<menuitem action='ReloadLIRC' />\n"
+#endif
 	   "<separator />\n"
 	   "<menuitem action='SetupWizards' />\n"
 	  "</menu>\n"
@@ -480,9 +496,12 @@ static GtkWidget *add_menu_ui (GtkUIManager *ui, int i)
 	  "<separator />\n"
 	  "<menu action='WindowMenu'>\n"
 	    "<menuitem action='Window50' />\n"
+	    "<menuitem action='Window75' />\n"
 	    "<menuitem action='Window100' />\n"
+	    "<menuitem action='Window150' />\n"
 	    "<menuitem action='Window200' />\n"
 	    "<menuitem action='WindowDynamic' />\n"
+	    "<menuitem action='WindowRevert' />\n"
 	    "<separator />\n"
 	    "<menuitem action='WindowAutosize' />\n"
 	    "<menuitem action='WindowMagLow' />\n"
@@ -514,6 +533,14 @@ static GtkWidget *add_menu_ui (GtkUIManager *ui, int i)
 	    "<menuitem action='Subtitles5' />\n"
 	    "<menuitem action='Subtitles6' />\n"
 	    "<menuitem action='Subtitles7' />\n"
+	    "<menuitem action='Subtitles8' />\n"
+	    "<menuitem action='Subtitles9' />\n"
+	    "<menuitem action='Subtitles10' />\n"
+	    "<menuitem action='Subtitles11' />\n"
+	    "<menuitem action='Subtitles12' />\n"
+	    "<menuitem action='Subtitles13' />\n"
+	    "<menuitem action='Subtitles14' />\n"
+	    "<menuitem action='Subtitles15' />\n"
 	  "</menu>\n"
 	  "<separator />\n"
 	  "<menuitem action='EngineLog' />\n"
@@ -581,9 +608,19 @@ GtkAction *find_video_size_action (gdouble factor)
 
 void scale_changed_cb (GtkWidget *widget, gdouble factor, gpointer data)
 {
-  logprintf ("gxine: received scale change event (%lf%%)\n", factor);
-  if (action_items.resize_factor && !find_video_size_action (factor))
+  if (factor != -1 && action_items.resize_factor)
   {
+    logprintf ("gxine: received scale change event (%lf%%)\n", factor);
+    GtkAction *action = find_video_size_action (factor);
+    if (action)
+    {
+      g_signal_handlers_block_by_func (action, js_Window_cb, NULL);
+      gtk_toggle_action_set_active ((GtkToggleAction *)action, TRUE);
+      do_pending_events ();
+      g_signal_handlers_unblock_by_func (action, js_Window_cb, NULL);
+      return;
+    }
+
     char *label = g_strdup_printf (_("_Other (%.1lf%%)"), factor);
     g_object_set ((GObject *)action_items.resize_factor, "label", label, NULL);
     free (label);
@@ -600,18 +637,20 @@ void scale_changed_cb (GtkWidget *widget, gdouble factor, gpointer data)
     if (changed)
       g_signal_handlers_unblock_by_func (changed->data, js_Window_cb, NULL);
   }
+  else
+    logprintf ("gxine: ignored scale change event (%lf%%)\n", factor);
 }
 
 void create_menus (GtkWidget *window)
 {
   GtkActionGroup *action;
-  static const int scale[] = { 50e+6, 100e+6, 200e+6 };
+  static const int scale[] = { 50e+6, 75e+6, 100e+6, 150e+6, 200e+6 };
 
   ui = ui_create_manager ("gxine", window);
   action = ui_get_action_group (ui);
 
   xine_cfg_entry_t entry;
-  int i;
+  unsigned int i;
 
   for (i = 0; i < G_N_ELEMENTS(toggle_prefs); ++i)
     if (toggle_prefs[i] &&
@@ -621,7 +660,7 @@ void create_menus (GtkWidget *window)
   if (!xine_config_lookup_entry (xine, "gui.fullscreen_toolbar", &entry))
     entry.num_value = 0;
   toggle_actions[TOGGLE_FULLSCREEN_TOOLBAR].is_active = (entry.num_value >> 1) & 1;
-    
+
 #define AG(group) (group), G_N_ELEMENTS((group))
   gtk_action_group_add_actions (action, AG(main_actions), NULL);
   gtk_action_group_add_toggle_actions (action, AG(toggle_actions), NULL);
@@ -670,6 +709,8 @@ void create_menus (GtkWidget *window)
   action_items.video_size = get_radio_group ("Window50");
   action_items.subtitles = get_radio_group ("Subtitles0");
 
+  action_items.keypad = get_action ("Keypad");
+
   if (xine_config_lookup_entry (xine, "gui.windowedmode_separate_toolbar",
 				&entry)
       && !entry.num_value)
