@@ -1,4 +1,4 @@
-/* 
+/*
  * Copyright (C) 2003-2004 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
@@ -108,7 +108,8 @@ void window_check_vis (gboolean force)
   {
     have_video = hv_new;
     have_audio = ha_new;
-    if (hv_new) {
+    if (hv_new)
+    {
       logprintf ("gxine: stream has video\n");
       vis_hide ((GtkVideo *)gtv, &audio_port);
 #ifdef USE_NET_WM_STATE_SHADED
@@ -211,8 +212,27 @@ static gboolean window_fs_toolbar_move (GdkEventConfigure *event)
   return FALSE;
 }
 
+static void window_fs_toolbar_set_geometry (void)
+{
+  GdkGeometry    cw_geom = {0};
+  if (gtv)
+  {
+    const GtkRequisition *size = gtk_video_get_fullscreen_geometry ((GtkVideo *)gtv);
+    cw_geom.max_width = cw_geom.min_width = size->width;
+  }
+  else
+    cw_geom.max_width = cw_geom.min_width = gdk_screen_width ();
+  cw_geom.max_height = cw_geom.min_height = -1;
+  cw_geom.win_gravity = GDK_GRAVITY_SOUTH_WEST;
+  gtk_window_set_geometry_hints
+    (GTK_WINDOW(fs_toolbar), fs_toolbar, &cw_geom,
+     GDK_HINT_POS | GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE | GDK_HINT_BASE_SIZE
+     | GDK_HINT_USER_POS);
+}
+
 static gboolean window_fs_toolbar_show_int (void)
 {
+  window_fs_toolbar_set_geometry ();
   window_fs_toolbar_move (NULL);
   window_show (fs_toolbar, NULL);
   gtk_window_stick (GTK_WINDOW(fs_toolbar));
@@ -253,18 +273,6 @@ void window_fs_toolbar_position (gboolean top)
   ui_set_status (UI_FS_TOOLBAR_POS);
 }
 
-static void window_fs_toolbar_set_geometry (void)
-{
-  GdkGeometry    cw_geom = {0};
-  cw_geom.max_width = cw_geom.min_width = gdk_screen_width ();
-  cw_geom.max_height = cw_geom.min_height = -1;
-  cw_geom.win_gravity = GDK_GRAVITY_SOUTH_WEST;
-  gtk_window_set_geometry_hints
-    (GTK_WINDOW(fs_toolbar), fs_toolbar, &cw_geom,
-     GDK_HINT_POS | GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE | GDK_HINT_BASE_SIZE
-     | GDK_HINT_USER_POS);
-}
-
 static void cw_show_cb (GtkWidget *widget)
 {
   if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION(action_items.fs_toolbar)))
@@ -417,17 +425,18 @@ static void fs_show_hide (GtkWidget *widget, gpointer show)
     gtk_widget_hide (widget);
 }
 
-static gboolean refocus_cb (gpointer widget)
+static gboolean refocus_cb (GtkWidget *widget)
 {
   if (GTK_WIDGET_VISIBLE (widget))
-    gtk_widget_grab_focus (widget);
+    XSetInputFocus (GDK_WINDOW_XDISPLAY (widget->window),
+		    GDK_WINDOW_XID (widget->window),
+		    RevertToParent, CurrentTime);
   return FALSE;
 }
 
 static gboolean appfocus_cb (GtkWidget *w, GdkEventFocus *e, gpointer d)
 {
-  g_print (_("Eek, toolbar has focus"));
-  g_idle_add (refocus_cb, app);
+  g_idle_add ((GSourceFunc)refocus_cb, app);
   return TRUE;
 }
 
@@ -581,15 +590,15 @@ xevent_filter_cb (GdkXEvent *gx, GdkEvent *e, gpointer d)
     if (gtk_video_is_fullscreen ((GtkVideo *) gtv))
     {
       Display *xd = GDK_WINDOW_XDISPLAY (((GtkWidget *) d)->window);
-#ifdef HAVE_XINERAMA
-      if (XineramaIsActive (xd))
-        return GDK_FILTER_REMOVE;
-#endif
       Window fw = None;
       int fr;
       XGetInputFocus (xd, &fw, &fr);
       GdkWindow *gw = gdk_window_lookup (fw);
+#ifdef HAVE_XINERAMA
+      if (gw == fs_toolbar->window || (!gw && !XineramaIsActive (xd)))
+#else
       if (!gw || gw == fs_toolbar->window)
+#endif
 	XSetInputFocus (xd, x->xany.window, RevertToParent, CurrentTime);
     }
     return GDK_FILTER_REMOVE;
@@ -613,8 +622,10 @@ static gboolean noskin_main_init_cb (void)
   if (wm_toolbar)
     g_signal_connect (G_OBJECT (wm_toolbar),
 		      "key-press-event", G_CALLBACK (keypress_cb), NULL);
-  g_signal_connect (G_OBJECT (fs_toolbar),
-		    "key-press-event", G_CALLBACK (keypress_cb), NULL);
+  g_object_connect (G_OBJECT (fs_toolbar),
+		    "signal::focus-in-event", G_CALLBACK (appfocus_cb), NULL,
+		    "signal::key-press-event", G_CALLBACK (keypress_cb), NULL,
+		    NULL);
   ui_set_status (xine_get_param (stream, XINE_PARAM_AUDIO_MUTE)
 		 ? UI_AUDIO_MUTE : UI_AUDIO_UNMUTE);
   gdk_window_set_events (app->window, GDK_ALL_EVENTS_MASK); /* seems to work */
@@ -744,7 +755,7 @@ void noskin_main_init (void)
   gtk_window_set_skip_pager_hint (W, TRUE);
   gtk_window_set_skip_taskbar_hint (W, TRUE);
 
-  window_fs_toolbar_set_geometry ();
+//  window_fs_toolbar_set_geometry ();
   drag_drop_setup (fs_toolbar, TRUE);
 
   w = widget_create_from_xml ("toolbar-fullscreen.xml", "toolbar_fs",
@@ -771,7 +782,7 @@ void noskin_main_init (void)
    * video widget
    */
 
-  gtv = gtk_video_new (xine, stream, xine_get_video_source (stream), 
+  gtv = gtk_video_new (xine, stream, xine_get_video_source (stream),
   	               video_driver_id, 320, 240,
                        0x04 /* press: button 2 */,
                        0x08 /* release: button 3 */);
@@ -835,7 +846,7 @@ void noskin_main_init (void)
     }
 
     g_object_connect (G_OBJECT (wm_toolbar),
-        "signal::grab-focus", G_CALLBACK (appfocus_cb), NULL,
+        "signal::focus-in-event", G_CALLBACK (appfocus_cb), NULL,
 	/*"signal::key-press-event", G_CALLBACK (keypress_cb), NULL,*/
 	"signal::map-event", G_CALLBACK (window_wm_toolbar_snap), NULL,
 	"signal::delete-event", G_CALLBACK (wm_close_cb), NULL,
@@ -881,6 +892,9 @@ void noskin_main_init (void)
 		     GTK_VIDEO_MIN_WIDTH, GTK_VIDEO_MIN_HEIGHT);
   gtk_widget_show_all (app);
 
+  gdk_window_set_decorations (app->window, GDK_DECOR_ALL | GDK_DECOR_MAXIMIZE);
+  gdk_window_set_functions (app->window, GDK_FUNC_ALL | GDK_FUNC_MAXIMIZE);
+
   if (wm_toolbar)
   {
     gtk_widget_show_all (wm_toolbar);
