@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004 the xine-project
+ * Copyright (C) 2004-2006 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -13,12 +13,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: player.c,v 1.56 2005/11/11 21:06:01 dsalt Exp $
+ * $Id: player.c,v 1.86 2006/04/08 21:34:50 dsalt Exp $
  *
- * player object 
+ * player object
  */
 
 #include "globals.h"
@@ -29,10 +29,15 @@
 #include <stdlib.h>
 #include <gtk/gtkradiomenuitem.h>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
 #include "globals.h"
 #include "menu.h"
 #include "ui.h"
 #include "utils.h"
+#include "preferences.h"
 #include "playlist.h"
 #include "gtkvideo.h"
 #include "snapshot.h"
@@ -56,32 +61,86 @@ static int             goto_pos, goto_time;
 static pthread_t       play_thread;
 static char           *cur_mrl = NULL, *cur_title = NULL;
 
-gboolean
-player_update_metadata (void)
+static __attribute__ ((noreturn)) void *play_exec (void *data)
 {
+  xine_cfg_entry_t entry;
+  char *hash, *mrl = NULL;
+
+  /*pthread_mutex_lock (&engine_lock);*/
+  pthread_mutex_lock (&widgets_update_lock);
   gdk_threads_enter();
-  infobar_show_metadata (infobars);
-  playlist_check_set_title ();
-  /*gdk_flush ();*/
-  gdk_threads_leave();
-  return FALSE;
-}
 
-static void *play_exec (void *queue_gen) {
-
-  pthread_mutex_lock (&engine_lock);
-
-  gdk_threads_enter();
   gxineinfo_clear (infobars);
   gxineinfo_update_line (infobars, 0, "%s", _("opening..."));
   gxineinfo_update_line (infobars, 1, "%s", cur_mrl);
-  gdk_threads_leave();
 
-  if (!xine_open (stream, cur_mrl))
+  if ((!xine_config_lookup_entry (xine, "gui.subtitle_autoload", &entry)
+       || entry.num_value)
+      && !strncmp (cur_mrl, "file:", 5)
+      && ((hash = strchr (cur_mrl, '#')) == NULL
+	  || (strncmp (hash, "#subtitle:", 10)
+	      && !strstr (hash, ";subtitle:"))))
+  {
+    char *name = cur_mrl + 5;
+    if (name)
+    {
+      unsigned int length = (strchr (name, '#') ? : name + strlen (name)) - name;
+      char file[length + 5]; /* allow for ".ext" + NUL */
+
+      unsigned int i = UINT_MAX, ext = 0;
+
+      /* URL decode (can't simply use g_filename_from_uri()...) */
+      while (++i < length)
+      {
+	if (name[i] == '%')
+	{
+	  char c[3] = { name[i+1], name[i+2], 0 };
+	  i += 2;
+	  file[ext] = (char) strtol (c, NULL, 16);
+	}
+	else
+	  file[ext] = name[i];
+	++ext;
+      }
+
+      /* find last '.' in leaf */
+      i = ext;
+      if (ext)
+	while (--ext && file[ext] != '.' && file[ext] != '/')
+	  /**/;
+      if (file[ext] == '/') /* if no '.' */
+	ext = i;
+
+      name = strrchr (name, '.');
+
+      static const char subs[][4] = { "sub", "srt", "asc", "smi", "ssa", "txt" };
+      for (i = 0; i < G_N_ELEMENTS (subs); ++i)
+	if (!name || strcmp (name + 1, subs[i]))
+	{
+	  struct stat st;
+	  sprintf (file + ext, ".%s", subs[i]);
+	  if (stat (file, &st) > -1 && S_ISREG (st.st_mode))
+	    break;
+	}
+
+      if (i < G_N_ELEMENTS (subs))
+      {
+	char *tmp = g_filename_to_uri (file, NULL, NULL);
+	char *uri = g_filename_to_uri (tmp + 7, NULL, NULL);
+	mrl = g_strconcat (cur_mrl, hash ? ";subtitle:" : "#subtitle:", uri + 7, NULL);
+	free (uri);
+	free (tmp);
+      }
+    }
+  }
+
+  gdk_threads_leave ();
+  if (!xine_open (stream, mrl ? : cur_mrl))
   {
     int error;
-    gdk_threads_enter();
-    switch (error = xine_get_error (stream)) {
+    gdk_threads_enter ();
+    switch (error = xine_get_error (stream))
+    {
     case XINE_ERROR_NO_INPUT_PLUGIN:
       display_error (FROM_XINE, _("The xine engine failed to start."),
 		     _("No input plugin was found.\n"
@@ -105,36 +164,37 @@ static void *play_exec (void *queue_gen) {
     }
 
     g_idle_add ((GSourceFunc)playlist_logo, NULL);
-    gdk_threads_leave();
+    gdk_threads_leave ();
     goto done;
   }
 
-  gdk_threads_enter();
+  gdk_threads_enter ();
   ui_set_status (playlist_showing_logo () ? UI_STOP : UI_PLAY);
   infobar_show_metadata (infobars);
-  gdk_threads_leave();
+  gdk_threads_leave ();
 
   xine_play (stream, goto_pos, goto_time);
   goto_pos = 0;
   goto_time = 0;
 
   /* correct for possible stream changes */
-  gdk_threads_enter();
+  gdk_threads_enter ();
   ui_set_status (playlist_showing_logo () ? UI_STOP : UI_PLAY);
-  // gtk_timeout_add (500, G_CALLBACK(player_update_metadata), NULL);
   infobar_show_metadata (infobars);
   playlist_check_set_title ();
 
 done:
-  gdk_threads_leave();
-  pthread_mutex_unlock (&engine_lock);
+  free (mrl);
+  gdk_threads_leave ();
+  pthread_mutex_unlock (&widgets_update_lock);
+  /*pthread_mutex_unlock (&engine_lock);*/
   pthread_exit (NULL);
-  return NULL;
 }
 
 void player_launch (const char *title, const char *mrl, int pos, int pos_time)
 {
   int err;
+  pthread_attr_t attr;
 
   goto_pos  = pos < 0 ? 0 : pos;
   goto_time = pos_time < 0 ? 0 : pos_time;
@@ -143,19 +203,28 @@ void player_launch (const char *title, const char *mrl, int pos, int pos_time)
   cur_title = title && !playlist_showing_logo () ? strdup (title) : NULL;
   cur_mrl = strdup (mrl);
 
-  gtk_video_unblank_screen (GTK_VIDEO (gtv));
+  gtk_video_unblank_screen ((GtkVideo *)gtv);
 
   /* start separate thread, so xine_open will not block ui */
-  if ((err = pthread_create (&play_thread,
-			     NULL, play_exec, NULL)) != 0) {
+  pthread_attr_init (&attr);
+  pthread_attr_setstacksize (&attr, 256 * 1024); /* probably overkill */
+  if ((err = pthread_create (&play_thread, &attr, play_exec, NULL)) != 0)
+  {
     display_error_modal (FROM_GXINE, _("Player failure"),
 			 _("Can't create new thread: %s\n"), strerror(err));
     abort();
   }
 
-  pthread_detach (play_thread);
+  if (!playlist_showing_logo ())
+    pthread_detach (play_thread);
+
+  pthread_attr_destroy (&attr);
 }
 
+void player_wait (void)
+{
+  pthread_join (play_thread, NULL);
+}
 
 static void set_mute (int mute)
 {
@@ -163,8 +232,14 @@ static void set_mute (int mute)
   xine_set_param (stream, XINE_PARAM_AUDIO_MUTE, mute);
 }
 
+static inline int calc_speed (int i)
+{
+  return i ? (1 << (i - 1)) : 0;
+}
+
 static void set_speed (int speed)
 {
+  xine_set_param (stream, XINE_PARAM_SPEED, speed);
   if (speed == XINE_SPEED_NORMAL)
     ui_set_status (UI_PLAY);
   else if (speed > XINE_SPEED_NORMAL)
@@ -173,8 +248,22 @@ static void set_speed (int speed)
     ui_set_status (UI_PLAY_SLOW);
   else
     ui_set_status (UI_PAUSE);
+}
 
-  xine_set_param (stream, XINE_PARAM_SPEED, speed);
+static int listen_speed (void *se, se_t *se_t, se_o_t *o, se_prop_t *prop,
+			 se_prop_read_t r)
+{
+  set_speed (calc_speed (r.i));
+  return FALSE;
+}
+
+static void get_speed (se_prop_t *prop, se_prop_read_t *value)
+{
+  int speed = xine_get_param (stream, XINE_PARAM_SPEED);
+  int bit = 0;
+  if (speed)
+    while (++bit, speed >>= 1);
+  value->i = bit;
 }
 
 static void set_zoom (int zoom)
@@ -197,13 +286,13 @@ static void set_zoom (int zoom)
 ACTION_SET(fullscreen)
 static void get_fullscreen (se_prop_t *prop, se_prop_read_t *value)
 {
-  value->i = gtk_video_is_fullscreen (GTK_VIDEO(gtv));
+  value->i = gtk_video_is_fullscreen ((GtkVideo *)gtv);
 }
 
 ACTION_SET(deinterlace)
 static void get_deinterlace (se_prop_t *prop, se_prop_read_t *value)
 {
-  value->i = gtk_video_get_use_post_plugins_deinterlace (GTK_VIDEO(gtv));
+  value->i = gtk_video_get_use_post_plugins_deinterlace ((GtkVideo *)gtv);
 }
 
 static void set_subtitles (int sub)
@@ -213,7 +302,7 @@ static void set_subtitles (int sub)
     GSList *menuitem = action_items.subtitles;
     int count = g_slist_length (menuitem) - 3;
     if (count >= sub)
-    {    
+    {
       while (menuitem && --count > sub)
 	menuitem = menuitem->next;
       if (menuitem)
@@ -245,203 +334,191 @@ static void set_aspect (int aspect)
 ACTION_SET(auto_resize)
 static void get_auto_resize (se_prop_t *prop, se_prop_read_t *value)
 {
-  value->i = gtk_video_get_auto_resize (GTK_VIDEO(gtv));
+  value->i = gtk_video_get_auto_resize ((GtkVideo *)gtv);
 }
 
 ACTION_SET(auto_rescale)
 static void get_auto_rescale (se_prop_t *prop, se_prop_read_t *value)
 {
-  value->i = gtk_video_get_auto_rescale (GTK_VIDEO(gtv));
+  value->i = gtk_video_get_auto_rescale ((GtkVideo *)gtv);
 }
 
 ACTION_SET(vo_postproc)
 static void get_vo_postproc (se_prop_t *prop, se_prop_read_t *value)
 {
-  value->i = gtk_video_get_use_post_plugins_video (GTK_VIDEO(gtv));
+  value->i = gtk_video_get_use_post_plugins_video ((GtkVideo *)gtv);
 }
 
 ACTION_SET(ao_postproc)
 static void get_ao_postproc (se_prop_t *prop, se_prop_read_t *value)
 {
-  value->i = gtk_video_get_use_post_plugins_audio (GTK_VIDEO(gtv));
+  value->i = gtk_video_get_use_post_plugins_audio ((GtkVideo *)gtv);
 }
 
+static void update_postproc_chain (const char *pref, const char *value)
+{
+  xine_cfg_entry_t entry;
+  xine_config_lookup_entry (xine, pref, &entry);
+  entry.str_value = (char *)value;
+  preferences_update_entry (&entry);
+}
+
+static JSBool set_postproc_chain (const char *func, const char *pref,
+				  JSContext *cx, JSObject *obj,
+				  uintN argc, jsval *argv, jsval *rval)
+{
+  se_log_fncall (func);
+  se_argc_check (1, func);
+  se_arg_is_string (0, func);
+  update_postproc_chain (pref,
+			 JS_GetStringBytes (JS_ValueToString (cx, argv[0])));
+  return JS_TRUE;
+}
+
+
 static JSBool set_vo_postproc_chain (JSContext *cx, JSObject *obj,
 				     uintN argc, jsval *argv, jsval *rval)
 {
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-
-  se_log_fncall ("vo_postproc.set_chain");
-  se_argc_check (1, "vo_postproc.set_chain");
-  se_arg_is_string (0, "vo_postproc.set_chain");
-
-  gtk_video_set_post_plugins_video
-    (GTK_VIDEO(gtv), JS_GetStringBytes (JS_ValueToString (cx, argv[0])));
-  return JS_TRUE;
+  return set_postproc_chain ("vo_postproc.set_chain", "gui.post_plugins.video",
+			     cx, obj, argc, argv, rval);
 }
 
 static JSBool set_ao_postproc_chain (JSContext *cx, JSObject *obj,
 				     uintN argc, jsval *argv, jsval *rval)
 {
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-
-  se_log_fncall ("ao_postproc.set_chain");
-  se_argc_check (1, "ao_postproc.set_chain");
-  se_arg_is_string (0, "ao_postproc.set_chain");
-
-  gtk_video_set_post_plugins_audio
-    (GTK_VIDEO(gtv), JS_GetStringBytes (JS_ValueToString (cx, argv[0])),
-     audio_port);
-  return JS_TRUE;
+  return set_postproc_chain ("ao_postproc.set_chain", "gui.post_plugins.audio",
+			     cx, obj, argc, argv, rval);
 }
 
-static JSBool get_time (JSContext *cx, JSObject *obj, uintN argc, 
-		     jsval *argv, jsval *rval) {
+static int listen_postproc_chain (void *pref, se_t *se, se_o_t *obj,
+				  se_prop_t *prop, se_prop_read_t value)
+{
+  update_postproc_chain (pref, value.s);
+  return 0;
+}
 
-  /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
+static JSBool get_time (JSContext *cx, JSObject *obj, uintN argc,
+		     jsval *argv, jsval *rval)
+{
   int   pos, pos_time, len;
-
   se_log_fncall ("get_time");
-
   xine_get_pos_length (stream, &pos, &pos_time, &len);
-
   *rval = INT_TO_JSVAL (pos_time);
-
   return JS_TRUE;
 }
 
 #ifdef WITH_DEPRECATED
 
-static JSBool js_get_speed (JSContext *cx, JSObject *obj, uintN argc, 
-			    jsval *argv, jsval *rval) {
-
-  /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
-  int   speed;
-
-  se_log_fncall ("get_speed");
-
-  speed = xine_get_param (stream, XINE_PARAM_SPEED);
-
-  *rval = INT_TO_JSVAL (speed);
-
+static JSBool js_get_speed (JSContext *cx, JSObject *obj, uintN argc,
+			    jsval *argv, jsval *rval)
+{
+  se_prop_read_t speed;
+  se_log_fncall_deprecated ("get_speed");
+  get_speed (NULL, &speed);
+  *rval = INT_TO_JSVAL (speed.i);
   return JS_TRUE;
 }
 
-static JSBool js_get_volume (JSContext *cx, JSObject *obj, uintN argc, 
-			    jsval *argv, jsval *rval) {
-
+static JSBool js_get_volume (JSContext *cx, JSObject *obj, uintN argc,
+			    jsval *argv, jsval *rval)
+{
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
   int   volume;
-
-  se_log_fncall ("get_volume");
-
+  se_log_fncall_deprecated ("get_volume");
   volume = xine_get_param (stream, XINE_PARAM_AUDIO_VOLUME);
-
   *rval = INT_TO_JSVAL (volume);
-
   return JS_TRUE;
 }
 
-static JSBool js_get_mute (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_get_mute (JSContext *cx, JSObject *obj, uintN argc,
 			   jsval *argv, jsval *rval)
 {
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
   int mute;
-  se_log_fncall ("get_mute");
+  se_log_fncall_deprecated ("get_mute");
   mute = xine_get_param (stream, XINE_PARAM_AUDIO_MUTE);
   *rval = BOOLEAN_TO_JSVAL (mute);
   return JS_TRUE;
 }
 
-static JSBool js_get_zoom (JSContext *cx, JSObject *obj, uintN argc, 
-			   jsval *argv, jsval *rval) {
-
+static JSBool js_get_zoom (JSContext *cx, JSObject *obj, uintN argc,
+			   jsval *argv, jsval *rval)
+{
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
   int   zoom;
-
-  se_log_fncall ("get_zoom");
-
+  se_log_fncall_deprecated ("get_zoom");
   zoom = xine_get_param (stream, XINE_PARAM_VO_ZOOM_X);
-
   *rval = INT_TO_JSVAL (zoom);
-
   return JS_TRUE;
 }
 
 #endif /* WITH_DEPRECATED */
 
-static JSBool controls_pause (JSContext *cx, JSObject *obj, uintN argc, 
-			      jsval *argv, jsval *rval) {
-
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  int   pause_state=-1;
+static JSBool controls_pause (JSContext *cx, JSObject *obj, uintN argc,
+			      jsval *argv, jsval *rval)
+{
+  int32 pause_state=-1;
 
   se_log_fncall_checkinit ("pause");
   se_argc_check_max (1, "pause");
 
-  if (argc==1) {
+  if (argc == 1)
+  {
     se_arg_is_int_or_bool (0, "pause");
     JS_ValueToInt32 (cx, argv[0], &pause_state);
   }
 
-  if (engine_mutex_trylock (se))
-    return JS_TRUE;
-
-  switch (pause_state) {
+  switch (pause_state)
+  {
   case 0: /* pause off */
-    ui_set_status (UI_PLAY);
-    xine_set_param (stream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
+    set_speed (XINE_SPEED_NORMAL);
     break;
   case 1: /* pause on */
-    ui_set_status (UI_PAUSE);
-    xine_set_param (stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
+    set_speed (XINE_SPEED_PAUSE);
     break;
   default: /* toggle */
     if (xine_get_param (stream, XINE_PARAM_SPEED) != XINE_SPEED_PAUSE)
-    {
-      ui_set_status (UI_PAUSE);
-      xine_set_param (stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
-    }
+      set_speed (XINE_SPEED_PAUSE);
     else
-    {
-      ui_set_status (UI_PLAY);
-      xine_set_param (stream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
-    }
+      set_speed (XINE_SPEED_NORMAL);
   }
 
-  pthread_mutex_unlock (&engine_lock);
-
   return JS_TRUE;
 }
 
-static JSBool controls_stop (JSContext *cx, JSObject *obj, uintN argc, 
-			     jsval *argv, jsval *rval) {
-
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-
+static JSBool controls_stop (JSContext *cx, JSObject *obj, uintN argc,
+			     jsval *argv, jsval *rval)
+{
   se_log_fncall_checkinit ("stop");
-
-  if (engine_mutex_trylock (se))
-    return JS_TRUE;
-
   xine_stop (stream);
+  playlist_logo (NULL);
+  ui_set_control_adjustment (Control_SEEKER, 0);
+  return JS_TRUE;
+}
 
-  playlist_logo ();
-
-  pthread_mutex_unlock (&engine_lock);
-
+static JSBool controls_eject (JSContext *cx, JSObject *obj, uintN argc,
+			      jsval *argv, jsval *rval)
+{
+  se_log_fncall_checkinit ("eject");
+  if (xine_get_status (stream) == XINE_STATUS_PLAY)
+    xine_stop (stream);
+  gdk_threads_leave (); /* need to do this to allow play_exec() to proceed */
+  playlist_logo (cx); /* doesn't matter so long as it's not NULL */
+  gdk_threads_enter ();
+  ui_set_control_adjustment (Control_SEEKER, 0);
+  int ret = xine_eject (stream);
+  *rval = BOOLEAN_TO_JSVAL (ret);
   return JS_TRUE;
 }
 
 #ifdef WITH_DEPRECATED
 
-static JSBool js_set_speed (JSContext *cx, JSObject *obj, uintN argc, 
-			    jsval *argv, jsval *rval) {
+static JSBool js_set_speed (JSContext *cx, JSObject *obj, uintN argc,
+			    jsval *argv, jsval *rval)
+{
+  int32 speed;
 
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  int   speed;
-
-  se_log_fncall ("set_speed");
+  se_log_fncall_deprecated ("set_speed");
 
   se_argc_check (1, "set_speed");
   se_arg_is_int (0, "set_speed");
@@ -450,21 +527,17 @@ static JSBool js_set_speed (JSContext *cx, JSObject *obj, uintN argc,
 
   if (speed<0)
     speed = 0;
+  set_speed (calc_speed (speed));
 
-  if (engine_mutex_trylock (se))
-    return JS_TRUE;
-  set_speed (speed);
-  pthread_mutex_unlock (&engine_lock);
   return JS_TRUE;
 }
 
-static JSBool js_set_volume (JSContext *cx, JSObject *obj, uintN argc, 
-			    jsval *argv, jsval *rval) {
+static JSBool js_set_volume (JSContext *cx, JSObject *obj, uintN argc,
+			    jsval *argv, jsval *rval)
+{
+  int32 volume;
 
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  int   volume;
-
-  se_log_fncall ("set_volume");
+  se_log_fncall_deprecated ("set_volume");
   se_argc_check (1, "set_volume");
   se_arg_is_int (0, "set_volume");
 
@@ -472,25 +545,18 @@ static JSBool js_set_volume (JSContext *cx, JSObject *obj, uintN argc,
 
   if (volume<0)
     volume = 0;
-
-  if (engine_mutex_trylock (se))
-    return JS_TRUE;
-
   ui_set_control_adjustment (Control_VOLUME, volume);
   ui_xine_set_param_from_adjustment (Control_VOLUME);
 
-  pthread_mutex_unlock (&engine_lock);
-
   return JS_TRUE;
 }
 
-static JSBool js_set_mute (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_set_mute (JSContext *cx, JSObject *obj, uintN argc,
 			   jsval *argv, jsval *rval)
 {
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
   JSBool mute;
 
-  se_log_fncall ("set_mute");
+  se_log_fncall_deprecated ("set_mute");
   se_argc_check_max (1, "set_mute");
 
   if (argc == 1)
@@ -501,106 +567,113 @@ static JSBool js_set_mute (JSContext *cx, JSObject *obj, uintN argc,
   else
     mute = !xine_get_param (stream, XINE_PARAM_AUDIO_MUTE);
 
-  if (engine_mutex_trylock (se))
-    return JS_TRUE;
   set_mute (mute);
-  pthread_mutex_unlock (&engine_lock);
+
   return JS_TRUE;
 }
 
-static JSBool js_set_zoom (JSContext *cx, JSObject *obj, uintN argc, 
-			   jsval *argv, jsval *rval) {
+static JSBool js_set_zoom (JSContext *cx, JSObject *obj, uintN argc,
+			   jsval *argv, jsval *rval)
+{
+  int32 zoom;
 
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  int   zoom;
-
-  se_log_fncall ("set_zoom");
+  se_log_fncall_deprecated ("set_zoom");
   se_argc_check (1, "set_zoom");
   se_arg_is_int (0, "set_zoom");
 
   JS_ValueToInt32 (cx, argv[0], &zoom);
-
-  se_log ("zoom is %d\n", zoom);
-
-  if (engine_mutex_trylock (se))
-    return JS_TRUE;
   set_zoom (zoom);
-  pthread_mutex_unlock (&engine_lock);
 
   return JS_TRUE;
 }
 
 #endif /* WITH_DEPRECATED */
 
-static JSBool js_set_video_size (JSContext *cx, JSObject *obj, uintN argc, 
-			   jsval *argv, jsval *rval) {
+static int zoomcb, zoomcb_id = 0;
 
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  int   zoom;
-  GtkAction *action;
-
-  se_log_fncall ("set_video_size");
-  se_argc_check (1, "set_video_size");
-  se_arg_is_int (0, "set_video_size");
-
-  JS_ValueToInt32 (cx, argv[0], &zoom);
-
-  if (zoom<GXINE_VO_ZOOM_MIN)
-    zoom = GXINE_VO_ZOOM_MIN;
-  else if (zoom>XINE_VO_ZOOM_MAX)
-    zoom = XINE_VO_ZOOM_MAX;
+static gboolean video_rescale_cb (gpointer unused)
+{
+  gtk_video_rescale ((GtkVideo *) gtv, zoomcb);
+  zoomcb_id = 0;
+  return FALSE;
+}
 
+static gboolean set_video_size_cb (gpointer unused)
+{
+  gdk_threads_enter ();
   /* find & activate an action item which corresponds to the requested scale */
-  action = find_video_size_action (zoom);
+  GtkAction *action = (zoomcb == -1) ? NULL : find_video_size_action (zoomcb);
   if (action)
     gtk_action_activate (action);
   else
   {
-    scale_changed_cb (NULL, zoom, NULL);
-    do_pending_events ();
-    gtk_video_rescale ((GtkVideo *) gtv, zoom);
+    scale_changed_cb (NULL, zoomcb, NULL);
+    zoomcb_id = g_idle_add ((GSourceFunc) video_rescale_cb, NULL);
   }
+  gdk_threads_leave ();
+  return FALSE;
+}
+
+static JSBool js_set_video_size (JSContext *cx, JSObject *obj, uintN argc,
+			   jsval *argv, jsval *rval)
+{
+  se_log_fncall ("set_video_size");
+  se_argc_check (1, "set_video_size");
+  se_arg_is_int (0, "set_video_size");
+
+  int32 zoom;
+  JS_ValueToInt32 (cx, argv[0], &zoom);
+
+  if (zoom < GXINE_VO_ZOOM_MIN && zoom != -1)
+    zoom = GXINE_VO_ZOOM_MIN;
+  else if (zoom > XINE_VO_ZOOM_MAX)
+    zoom = XINE_VO_ZOOM_MAX;
+
+  zoomcb = zoom;
+  if (zoomcb_id)
+    g_source_remove (zoomcb_id);
+  zoomcb_id = g_idle_add ((GSourceFunc) set_video_size_cb, NULL);
 
   return JS_TRUE;
 }
 
 #ifdef WITH_DEPRECATED
 
-static JSBool js_set_fullscreen (JSContext *cx, JSObject *obj, uintN argc, 
-				 jsval *argv, jsval *rval) {
+static JSBool js_set_fullscreen (JSContext *cx, JSObject *obj, uintN argc,
+				 jsval *argv, jsval *rval)
+{
+  int32 fs;
 
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  int   fs;
-
-  se_log_fncall ("set_fullscreen");
+  se_log_fncall_deprecated ("set_fullscreen");
   se_argc_check_max (1, "set_fullscreen");
 
-  if (argc==1) {
+  if (argc == 1)
+  {
     se_arg_is_int_or_bool (0, "set_fullscreen");
     JS_ValueToInt32 (cx, argv[0], &fs);
-  } else {
-    fs = !gtk_video_is_fullscreen (GTK_VIDEO(gtv));
   }
+  else
+    fs = !gtk_video_is_fullscreen ((GtkVideo *)gtv);
 
   set_fullscreen (fs);
   return JS_TRUE;
 }
 
-static JSBool js_set_deinterlace (JSContext *cx, JSObject *obj, uintN argc, 
-				 jsval *argv, jsval *rval) {
+static JSBool js_set_deinterlace (JSContext *cx, JSObject *obj, uintN argc,
+				 jsval *argv, jsval *rval)
+{
+  int32 di;
 
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  int   di;
-
-  se_log_fncall ("set_deinterlace");
+  se_log_fncall_deprecated ("set_deinterlace");
   se_argc_check_max (1, "set_deinterlace");
 
-  if (argc==1) {
+  if (argc == 1)
+  {
     se_arg_is_int_or_bool (0, "set_deinterlace");
     JS_ValueToInt32 (cx, argv[0], &di);
-  } else {
-    di = !gtk_video_get_use_post_plugins_deinterlace (GTK_VIDEO(gtv));
   }
+  else
+    di = !gtk_video_get_use_post_plugins_deinterlace ((GtkVideo *)gtv);
 
   set_deinterlace (di);
   return JS_TRUE;
@@ -610,9 +683,9 @@ static JSBool js_set_postproc_video (JSContext *cx, JSObject *obj,
 				     uintN argc, jsval *argv, jsval *rval)
 {
   se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  int di;
+  int32 di;
 
-  se_log_fncall ("set_postproc_video");
+  se_log_fncall_deprecated ("set_postproc_video");
   se_argc_check_max (1, "set_postproc_video");
 
   if (argc == 1)
@@ -620,11 +693,7 @@ static JSBool js_set_postproc_video (JSContext *cx, JSObject *obj,
     if (JSVAL_IS_INT (argv[0]))
       JS_ValueToInt32 (cx, argv[0], &di);
     else if (JSVAL_IS_STRING (argv[0]))
-    {
-      gtk_video_set_post_plugins_video
-        (GTK_VIDEO(gtv), JS_GetStringBytes (JS_ValueToString (cx, argv[0])));
-      return JS_TRUE;
-    }
+      return set_vo_postproc_chain (cx, obj, argc, argv, rval);
     else
     {
       se->print_cb (se->print_cb_data,
@@ -633,7 +702,7 @@ static JSBool js_set_postproc_video (JSContext *cx, JSObject *obj,
       return JS_TRUE;
     }
   } else
-    di = !gtk_video_get_use_post_plugins_video (GTK_VIDEO(gtv));
+    di = !gtk_video_get_use_post_plugins_video ((GtkVideo *)gtv);
 
   set_vo_postproc (di);
   return JS_TRUE;
@@ -643,9 +712,9 @@ static JSBool js_set_postproc_audio (JSContext *cx, JSObject *obj,
 				     uintN argc, jsval *argv, jsval *rval)
 {
   se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  int di;
+  int32 di;
 
-  se_log_fncall ("set_postproc_audio");
+  se_log_fncall_deprecated ("set_postproc_audio");
   se_argc_check_max (1, "set_postproc_audio");
 
   if (argc == 1)
@@ -653,12 +722,7 @@ static JSBool js_set_postproc_audio (JSContext *cx, JSObject *obj,
     if (JSVAL_IS_INT (argv[0]))
       JS_ValueToInt32 (cx, argv[0], &di);
     else if (JSVAL_IS_STRING (argv[0]))
-    {
-      gtk_video_set_post_plugins_audio
-	(GTK_VIDEO(gtv), JS_GetStringBytes (JS_ValueToString (cx, argv[0])),
-	 audio_port);
-      return JS_TRUE;
-    }
+      return set_ao_postproc_chain (cx, obj, argc, argv, rval);
     else
     {
       se->print_cb (se->print_cb_data,
@@ -667,27 +731,27 @@ static JSBool js_set_postproc_audio (JSContext *cx, JSObject *obj,
       return JS_TRUE;
     }
   } else
-    di = !gtk_video_get_use_post_plugins_audio (GTK_VIDEO(gtv));
+    di = !gtk_video_get_use_post_plugins_audio ((GtkVideo *)gtv);
 
   set_ao_postproc (di);
   return JS_TRUE;
 }
 
 static JSBool js_set_auto_resize (JSContext *cx, JSObject *obj, uintN argc,
-				 jsval *argv, jsval *rval) {
+				 jsval *argv, jsval *rval)
+{
+  int32 ar;
 
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  int   ar;
-
-  se_log_fncall ("set_auto_resize");
+  se_log_fncall_deprecated ("set_auto_resize");
   se_argc_check_max (1, "set_auto_resize");
 
-  if (argc==1) {
+  if (argc == 1)
+  {
     se_arg_is_int_or_bool (0, "set_auto_resize");
     JS_ValueToInt32 (cx, argv[0], &ar);
-  } else {
-    ar = !gtk_video_get_auto_resize (GTK_VIDEO(gtv));
   }
+  else
+    ar = !gtk_video_get_auto_resize ((GtkVideo *)gtv);
 
   set_auto_resize (ar);
   return JS_TRUE;
@@ -696,36 +760,37 @@ static JSBool js_set_auto_resize (JSContext *cx, JSObject *obj, uintN argc,
 static JSBool js_set_auto_rescale (JSContext *cx, JSObject *obj, uintN argc,
 				   jsval *argv, jsval *rval)
 {
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  int   ar;
+  int32 ar;
 
-  se_log_fncall ("set_auto_rescale");
+  se_log_fncall_deprecated ("set_auto_rescale");
   se_argc_check_max (1, "set_auto_rescale");
 
-  if (argc==1) {
+  if (argc == 1)
+  {
     se_arg_is_int_or_bool (0, "set_auto_rescale");
     JS_ValueToInt32 (cx, argv[0], &ar);
-  } else {
-    ar = !gtk_video_get_auto_rescale (GTK_VIDEO(gtv));
   }
+  else
+    ar = !gtk_video_get_auto_rescale ((GtkVideo *)gtv);
 
   set_auto_rescale (ar);
   return JS_TRUE;
 }
 
-static JSBool js_set_aspect (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_set_aspect (JSContext *cx, JSObject *obj, uintN argc,
 			     jsval *argv, jsval *rval)
 {
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  int aspect;
+  int32 aspect;
 
-  se_log_fncall ("set_aspect");
+  se_log_fncall_deprecated ("set_aspect");
   se_argc_check_max (1, "set_aspect");
 
-  if (argc==1) {
+  if (argc==1)
+  {
     se_arg_is_int (0, "set_aspect");
     JS_ValueToInt32 (cx, argv[0], &aspect);
-  } else
+  }
+  else
     aspect = xine_get_param (stream, XINE_PARAM_VO_ASPECT_RATIO) + 1;
 
   set_aspect (aspect);
@@ -734,10 +799,9 @@ static JSBool js_set_aspect (JSContext *cx, JSObject *obj, uintN argc,
 
 #endif /* WITH_DEPRECATED */
 
-static JSBool js_snapshot (JSContext *cx, JSObject *obj, uintN argc, 
-			   jsval *argv, jsval *rval) {
-
-  se_t     *se = (se_t *) JS_GetContextPrivate(cx);
+static JSBool js_snapshot (JSContext *cx, JSObject *obj, uintN argc,
+			   jsval *argv, jsval *rval)
+{
   JSString *str;
   char     *fname = NULL;
   int32	    scale = -1, blend = -1;
@@ -759,97 +823,23 @@ static JSBool js_snapshot (JSContext *cx, JSObject *obj, uintN argc,
     fname = JS_GetStringBytes (str);
   }
 
-  if (engine_mutex_trylock (se))
-    return JS_TRUE;
-
   make_snapshot (fname, scale, blend);
 
-  pthread_mutex_unlock (&engine_lock);
-
   return JS_TRUE;
 }
 
-/**************************************************************
- *
- * player.controls class & object
- *
- * adapted from http://users.skynet.be/saw/SpiderMonkey.htm
- *
- **************************************************************/
-
-/*
- * controls object properties
- */
-
-#if 0
-int controls_prop_cb (void *user_data, char *property, int *num) {
-  /* FIXME */
-  return 1;
-}
-
-#define CONTROLS_PROP_AUDIO_LANGUAGE_COUNT          0
-#define CONTROLS_PROP_CURRENT_AUDIO_LANGUAGE        1
-#define CONTROLS_PROP_CURRENT_AUDIO_LANGUAGE_INDEX  2
-#define CONTROLS_PROP_CURRENT_ITEM                  3
-#define CONTROLS_PROP_CURRENT_MARKER                4
-#define CONTROLS_PROP_CURRENT_POSITION              5
-#define CONTROLS_PROP_CURRENT_POSITION_STRING       6
-#define CONTROLS_PROP_CURRENT_POSITION_TIMECODE     7
-#define CONTROLS_PROP_IS_AVAILABLE                  8
-
-JSBool controls_JSGetProperty (JSContext *cx, JSObject *obj, jsval id, jsval *vp) {
-
-  se_log_fncall ("get_property");
-
-  if (JSVAL_IS_STRING (id)) {
-    JSString *str;
-    char     *prop;
-
-    se_log ("id is a string\n");
-
-    str = JS_ValueToString (cx, id);
-    prop = JS_GetStringBytes (str);
-
-    se_log ("id = '%s'\n", JS_GetStringBytes (str));
-
-    if (!strcmp (prop, "audioLanguageCount")) {
-      *vp = INT_TO_JSVAL (1);
-    } else if (!strcmp (prop, "currentPosition")) {
-      *vp = INT_TO_JSVAL (0);
-    }
-  }
-#if 0
-  if (JSVAL_IS_INT(id)) 
-    {
-      Customer *priv = (Customer *) JS_GetPrivate(cx, obj);
-      switch(JSVAL_TO_INT(id))
-	{
-	case name_prop:
-	  
-	  break;
-	case age_prop:
-	  *vp = INT_TO_JSVAL(priv->getCustomer()->GetAge());
-	  break;
-	}
-    }
-#endif
-  return JS_TRUE;
-}
-#endif
-  
-static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc, 
-			     jsval *argv, jsval *rval) {
-
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  char *mrl = NULL;
-  int  pos, pos_time;
 
+static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
+			     jsval *argv, jsval *rval)
+{
   se_log_fncall_checkinit ("play");
   se_argc_check_max (3, "play");
 
-  pos = -1; pos_time = -1;
+  char *mrl = NULL;
+  int32 pos = -1, pos_time = -1;
 
-  switch (argc) {
+  switch (argc)
+  {
   case 0:
     break;
 
@@ -865,7 +855,7 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
     }
     break;
 
-  case 2: 
+  case 2:
     {
       se_arg_is_int (0, "play");
       se_arg_is_int (1, "play");
@@ -873,7 +863,7 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
       JS_ValueToInt32 (cx, argv[0], &pos);
       JS_ValueToInt32 (cx, argv[1], &pos_time);
       mrl = NULL;
-      se_log ("playing from %d, %d\n", pos, pos_time); 
+      se_log ("playing from %d, %d\n", pos, pos_time);
     }
     break;
 
@@ -896,9 +886,6 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
   if (pos > 0)
     pos *= 655;
 
-  if (engine_mutex_trylock (se))
-    return JS_TRUE;
-
   if (mrl)
     playlist_play_from (playlist_add_mrl (mrl, -1), pos, pos_time);
   else
@@ -917,22 +904,18 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
       default:
 	if (pos >= 0 || pos_time >= 0)
 	{
-	  gtk_video_unblank_screen (GTK_VIDEO (gtv));
+	  gtk_video_unblank_screen ((GtkVideo *)gtv);
 	  xine_play (stream, pos < 0 ? 0 : pos, pos_time < 0 ? 0 : pos_time);
         }
 	break;
       }
       play_item_dispose (item);
     } else
-      logprintf (_("script_engine: error, no valid play item available\n"));
-  } 
-
-  if (xine_get_param (stream, XINE_PARAM_SPEED) != XINE_SPEED_NORMAL) {
-    ui_set_status (UI_PLAY);
-    xine_set_param (stream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
+      logprintf (_("script_engine: error: no valid play item available\n"));
   }
 
-  pthread_mutex_unlock (&engine_lock);
+  if (xine_get_param (stream, XINE_PARAM_SPEED) != XINE_SPEED_NORMAL)
+    set_speed (XINE_SPEED_NORMAL);
 
   return JS_TRUE;
 }
@@ -941,16 +924,13 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
  * (Useful on some keyboards with CD/DVD control keys.)
  */
 static JSBool controls_play_pause (JSContext *cx, JSObject *obj, uintN argc,
-                                  jsval *argv, jsval *rval) {
-
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-
+                                  jsval *argv, jsval *rval)
+{
   se_log_fncall_checkinit ("play_pause");
   se_argc_check_max (1, "play_pause");
 
-  if (argc==1) {
+  if (argc == 1)
     se_arg_is_int_or_bool (0, "play_pause");
-  }
 
   switch (xine_get_status (stream))
   {
@@ -965,8 +945,6 @@ static JSBool controls_play_pause (JSContext *cx, JSObject *obj, uintN argc,
 static JSBool js_is_live_stream (JSContext *cx, JSObject *obj, uintN argc,
                                  jsval *argv, jsval *rval)
 {
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-
   *rval = JSVAL_FALSE;
 
   se_log_fncall ("is_live_stream");
@@ -984,7 +962,8 @@ int player_live_stream (void)
 
   return (xine_get_status (stream) == XINE_STATUS_PLAY) &&
 	 (xine_get_pos_length (stream, &pos_stream, &pos_time, &length_time)) &&
-	 (pos_stream | length_time) == 0;
+	 length_time == 0 &&
+	 !xine_get_stream_info (stream, XINE_STREAM_INFO_SEEKABLE);
 }
 
 const char *player_get_cur_title (void)
@@ -998,23 +977,20 @@ const char *player_get_cur_mrl (void)
 }
 
 
-static xine_audio_port_t *load_audio_out_driver (void) {
- 
+static xine_audio_port_t *load_audio_out_driver (void)
+{
   xine_audio_port_t      *audio_port;
- 
-  if (!audio_driver_id) {
- 
-    char **driver_ids = (char **) xine_list_audio_output_plugins (xine) ;
-    char **choices;
-    int i;
- 
-    choices = malloc (sizeof (char *) * 100);
-    choices[0] = "auto"; i=0;
-    while (driver_ids[i]) {
-      choices[i+1] = strdup(driver_ids[i]);
-      i++;
-    }
-    choices[i+1]=0;
+
+  if (!audio_driver_id)
+  {
+    char **driver_ids = (char **) xine_list_audio_output_plugins (xine);
+    char **choices = malloc (sizeof (char *) * 100);
+    choices[0] = "auto";
+
+    int i = -1;
+    while (driver_ids[++i])
+      choices[i + 1] = strdup(driver_ids[i]);
+    choices[i + 1]=0;
 
     /* try to init audio with stored information */
     i = xine_config_register_enum (xine,
@@ -1024,19 +1000,19 @@ static xine_audio_port_t *load_audio_out_driver (void) {
                                    NULL, 10, NULL, CONFIG_DATA_NONE);
     audio_driver_id = choices[i];
   }
- 
+
   if (!strcmp (audio_driver_id, "null"))
     return NULL;
- 
-  if (strcmp (audio_driver_id, "auto")) {
- 
+
+  if (strcmp (audio_driver_id, "auto"))
+  {
     audio_port = xine_open_audio_driver (xine, audio_driver_id, NULL);
     if (audio_port)
       return audio_port;
     else
-      fprintf (stderr, _("audio driver %s failed\n"), audio_driver_id);
+      g_printerr (_("audio driver %s failed\n"), audio_driver_id);
   }
- 
+
   /* autoprobe */
   return xine_open_audio_driver (xine, NULL, NULL);
 }
@@ -1048,7 +1024,7 @@ void player_init (void)
 /*
   se_o_t            *controls_obj;
  */
-  
+
   cur_mrl = NULL;
 
   /* load drivers, create xine stream */
@@ -1059,7 +1035,7 @@ void player_init (void)
   stream = xine_stream_new (xine, audio_port, video_port);
 
   /*
-   * set up script engine objects and functions 
+   * set up script engine objects and functions
    */
 
 /*
@@ -1076,7 +1052,7 @@ void player_init (void)
        *     VARIABLE[=TYPE] or METHOD([PARAMETER TYPE,...])
        *   To the right, descriptive text
        */
-      { "av_speed", N_("v=int, min, max; playback speed"), XINE_PARAM_SPEED, NULL, NULL, XINE_SPEED_PAUSE, XINE_SPEED_FAST_4 },
+      { "av_speed", N_("v=int, min, max; playback speed"), 0, get_speed, set_speed, XINE_SPEED_PAUSE, XINE_SPEED_FAST_4, FALSE, { listen_speed } },
       { "vo_aspect", N_("v=int; aspect ratio"), XINE_PARAM_VO_ASPECT_RATIO, NULL, set_aspect, 0, XINE_VO_ASPECT_NUM_RATIOS - 1, TRUE },
       { "vo_subtitle", N_("v=int, min, max; subtitle channel"), XINE_PARAM_SPU_CHANNEL, NULL, set_subtitles, -2, GXINE_MAX_SPU_CHANNEL },
       { "vo_zoom", N_("v=int, min, max; video zoom (within window)"), XINE_PARAM_VO_ZOOM_X, NULL, set_zoom, XINE_VO_ZOOM_MIN, XINE_VO_ZOOM_MAX },
@@ -1088,8 +1064,8 @@ void player_init (void)
       { "vo_auto_rescale", N_("v=bool, toggle(): double size of low-resolution video"), 0, get_auto_rescale, set_auto_rescale },
       { "vo_fullscreen", N_("v=bool, toggle(): full-screen mode"), 0, get_fullscreen, set_fullscreen },
       { "vo_deinterlace", N_("v=bool, toggle(): enable deinterlacing"), 0, get_deinterlace, set_deinterlace },
-      { "ao_postproc", N_("v=bool, toggle(): audio post-processing; set_chain(string): plugin chain"), 0, get_ao_postproc, set_ao_postproc },
-      { "vo_postproc", N_("v=bool, toggle(): video post-processing; set_chain(string): plugin chain"), 0, get_vo_postproc, set_vo_postproc },
+      { "ao_postproc", N_("v=bool, toggle(): audio post-processing; chain=string: plugin chain"), 0, get_ao_postproc, set_ao_postproc },
+      { "vo_postproc", N_("v=bool, toggle(): video post-processing; chain=string: plugin chain"), 0, get_vo_postproc, set_vo_postproc },
       { NULL }
     };
     static const ui_property_t zoom_subprops[] = {
@@ -1098,18 +1074,27 @@ void player_init (void)
       { NULL }
     };
 
-    ui_create_properties (int_props, NULL, SE_TYPE_INT, NULL);
-    ui_create_properties (bool_props, NULL, SE_TYPE_BOOL, NULL);
+    ui_create_properties (int_props, NULL, SE_TYPE_INT);
+    ui_create_properties (bool_props, NULL, SE_TYPE_BOOL);
     ui_create_properties (zoom_subprops, se_find_object (gse, NULL, "vo_zoom"),
-			  SE_TYPE_INT, NULL);
-
-    /* a few extras - chain definition should be done via the GUI */
-    se_defun (gse, se_find_object (gse, NULL, "ao_postproc"), "set_chain",
-	      set_ao_postproc_chain, 0, 0, SE_GROUP_HIDDEN, NULL, NULL);
-    se_defun (gse, se_find_object (gse, NULL, "vo_postproc"), "set_chain",
-	      set_vo_postproc_chain, 0, 0, SE_GROUP_HIDDEN, NULL, NULL);
+			  SE_TYPE_INT);
   }
 
+  /* a few extras - chain definition should be done via the GUI */
+  se_o_t *obj = se_find_object (gse, NULL, "ao_postproc");
+  se_defun (gse, obj, "set_chain",
+	    set_ao_postproc_chain, 0, 0, SE_GROUP_HIDDEN, NULL, NULL);
+  se_prop_create_xine_id (gse, obj, "chain", "gui.post_plugins.audio");
+  se_prop_add_listener (gse, obj, "chain",
+			listen_postproc_chain, "gui.post_plugins.audio");
+
+  obj = se_find_object (gse, NULL, "vo_postproc");
+  se_defun (gse, obj, "set_chain",
+	    set_vo_postproc_chain, 0, 0, SE_GROUP_HIDDEN, NULL, NULL);
+  se_prop_create_xine_id (gse, obj, "chain", "gui.post_plugins.video");
+  se_prop_add_listener (gse, obj, "chain",
+			listen_postproc_chain, "gui.post_plugins.video");
+
   /*
    * hook up global xine functions for convenience
    */
@@ -1126,9 +1111,12 @@ void player_init (void)
   	SE_GROUP_ENGINE, N_("[bool]"), NULL },
       { "stop", controls_stop, 0, 0,
 	SE_GROUP_ENGINE, NULL, NULL },
+      { "eject", controls_eject, 0, 0,
+	SE_GROUP_ENGINE, NULL, NULL },
       { "play_pause", controls_play_pause, 0, 0,
 	SE_GROUP_ENGINE, N_("[bool]"), NULL },
       { "snapshot", js_snapshot, 0, 0,
+        /* these correspond to the options in the snapshot-save dialogue box */
 	SE_GROUP_ENGINE, N_("[file name [, scale, blend]]"), NULL },
 
       { "set_video_size", js_set_video_size, 0, 0,
@@ -1176,7 +1164,7 @@ void player_init (void)
 
   {
     static char *window_size_labels[] = {
-      N_("50%"), N_("100%"), N_("200%"),
+      N_("50%"), N_("75%"), N_("100%"), N_("150%"), N_("200%"),
       NULL
     };
     xine_config_register_enum
@@ -1189,5 +1177,10 @@ void player_init (void)
        _("This affects video streams whose dimensions are at most ⅓ "
 	 "of those of the display."),
        0, NULL, NULL);
+    xine_config_register_bool
+      (xine, "gui.subtitle_autoload", 1,
+       _("Subtitle autoloading"),
+       _("Automatically load subtitles if they exist."),
+       0, NULL, NULL);
   }
 }
