@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2005 the xine-project
+ * Copyright (C) 2004-2006 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -13,10 +13,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: post.c,v 1.35 2005/11/02 01:20:57 dsalt Exp $
+ * $Id: post.c,v 1.45 2006/04/09 14:27:29 dsalt Exp $
  *
  * post-plugin handling
  */
@@ -37,6 +37,7 @@
 
 #include "post.h"
 #include "gtkvideo.h"
+#include "preferences.h"
 #include "ui.h"
 #include "utils.h"
 
@@ -70,7 +71,7 @@ static gxine_chain_t chains[] = {
   { POST_CHAIN_DEINTERLACE }, { POST_CHAIN_VIDEO }, { POST_CHAIN_AUDIO }
 };
 
-static const char *cfgname[] = {
+static const char *const cfgname[] = {
   "gui.post_plugins.deinterlace",
   "gui.post_plugins.video",
   "gui.post_plugins.audio"
@@ -89,6 +90,8 @@ static GtkTooltips *tips = NULL;
 static xine_video_port_t *vo_none = NULL;
 static xine_audio_port_t *ao_none = NULL;
 
+extern const char *xine_get_post_plugin_description () __attribute__((weak));
+
 static char *
 post_parse_parameters (xine_post_t *plugin, const char *paramstr)
 {
@@ -167,14 +170,12 @@ post_parse_parameters (xine_post_t *plugin, const char *paramstr)
 	  goto next_param;
 
 	case POST_PARAM_TYPE_STRING:
-	  fprintf (stderr,
-		   "gxine: post-parameter type %s not yet supported\n",
+	  g_printerr (_("gxine: post-parameter type %s not yet supported\n"),
 		   "STRING");
 	  goto next_param;
 
 	case POST_PARAM_TYPE_STRINGLIST:
-	  fprintf (stderr,
-		   "gxine: post-parameter type %s not yet supported\n",
+	  g_printerr (_("gxine: post-parameter type %s not yet supported\n"),
 		   "STRINGLIST");
 	  goto next_param;
 
@@ -185,8 +186,7 @@ post_parse_parameters (xine_post_t *plugin, const char *paramstr)
 	  goto next_param;
 
 	default:
-	  fprintf (stderr,
-		   "gxine: post-parameter type %s not yet supported\n",
+	  g_printerr (_("gxine: post-parameter type %s not yet supported\n"),
 		   "???");
 	  goto next_param;
 	}
@@ -498,6 +498,7 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
   g_signal_connect (G_OBJECT(info->dbox), "response",
 		    G_CALLBACK(post_config_post_response), info);
   gtk_dialog_set_default_response (GTK_DIALOG (info->dbox), GTK_RESPONSE_OK);
+  ui_add_undo_response (info->dbox, NULL);
   free (title);
   table = gtk_table_new (1, 2, FALSE);
   gtk_box_pack_start (GTK_BOX(GTK_DIALOG(info->dbox)->vbox), table,
@@ -568,10 +569,10 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 	sprintf (contents, "%.*s", param->size, (char *)param_p);
 	gtk_entry_set_text (GTK_ENTRY(widget), contents);
 	g_object_set_data (G_OBJECT(widget), "post", info);
-	g_signal_connect (G_OBJECT(widget), "insert-text",
-			  G_CALLBACK(text_insert_cb), NULL);
-	g_signal_connect (G_OBJECT(widget), "focus-out-event",
-			  G_CALLBACK(char_entry_cb), (gpointer)param);
+	g_object_connect (G_OBJECT(widget),
+		"signal::insert-text", G_CALLBACK(text_insert_cb), NULL,
+		"signal::focus-out-event", G_CALLBACK(char_entry_cb), (gpointer)param,
+		NULL);
       }
       break;
 
@@ -607,7 +608,7 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
     size_t p = strlen (helptext);
 
     /* lose trailing whitespace (and control codes) */
-    while (p >= 0 && helptext[p] < 33)
+    while (helptext[p] < 33)
       --p;
     helptext[p + 1] = 0;
 
@@ -619,19 +620,37 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 
       gtk_container_add (GTK_CONTAINER(window), help);
       gtk_text_view_set_editable ((GtkTextView *)help, FALSE);
-      gtk_text_view_set_wrap_mode ((GtkTextView *)help, 3);
-	/* earlier GTK+2.2 apparently doesn't define GTK_WRAP_WORD_CHAR */
 
       /* use the normal fg/bg colours, rather than the text display colours */
       g_signal_connect (G_OBJECT(help), "realize",
 			G_CALLBACK(help_realise_cb), NULL);
 
-      /* use normal font or, if there's at least one tab, monospaced;
+      /* use normal font or, if there are any tabs or no long lines,
+       * a monospaced font (it's better to err in this direction);
        * either way, reduce the size a little
        */
-      font = strchr (helptext, '\t')
-             ? pango_font_description_from_string ("mono")
-	     : pango_font_description_copy_static (help->style->font_desc);
+      gboolean mono = TRUE, wrap = FALSE;
+      char *lf1 = helptext - 1;
+      while (lf1)
+      {
+	char *lf2 = strchr (lf1 + 1, '\n');
+	if ((lf2 ? lf2 : lf1 + strlen (lf1)) - lf1 > 82)
+	{
+	  mono = FALSE;
+	  wrap = TRUE;
+	  break;
+	}
+	lf1 = lf2;
+      }
+      if (strchr (helptext, '\t'))
+	mono = TRUE;
+
+      gtk_text_view_set_wrap_mode ((GtkTextView *)help,
+				   wrap ? GTK_WRAP_WORD_CHAR : GTK_WRAP_NONE);
+
+      font = mono ? pango_font_description_from_string ("mono")
+		  : pango_font_description_copy_static (help->style->font_desc);
+
       p = pango_font_description_get_size (help->style->font_desc);
       pango_font_description_set_size (font, p * 5 / 6);
       gtk_widget_modify_font (help, font);
@@ -641,11 +660,12 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 
       gtk_text_buffer_set_text (buffer, helptext, -1);
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(window),
-				      GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-      gtk_widget_set_size_request (window, 1, 16);
+				      wrap ? GTK_POLICY_NEVER : GTK_POLICY_AUTOMATIC,
+				      GTK_POLICY_AUTOMATIC);
+
+      gtk_widget_set_size_request (window, 1, -1);
       gtk_box_pack_end (GTK_BOX(GTK_DIALOG(info->dbox)->vbox), window,
 			TRUE, TRUE, 2);
-
     }
     free (helptext);
 
@@ -709,7 +729,7 @@ reconfigure_plugins (gxine_chain_t *chain)
   }
 
   entry.str_value = params ? : "";
-  xine_config_update_entry (xine, &entry);
+  preferences_update_entry (&entry);
 
   free (params);
 }
@@ -745,7 +765,8 @@ post_config_post_chain_set_plugin (GtkOptionMenu *menu, gxine_post_t *info)
   if (!newplugin)
   {
     display_error_modal (FROM_XINE, _("Plugin error"),
-			 _("Couldn't initialise plugin '%s' for configuration"),
+	/* problem - we can't instantiate the plugin so we can't configure it */
+			 _("Couldn't initialise plugin ‘%s’ for configuration"),
 			 name);
     if (info->name && strcmp (name, info->name))
     {
@@ -772,6 +793,12 @@ post_config_post_chain_set_plugin (GtkOptionMenu *menu, gxine_post_t *info)
   info->plugin = newplugin;
   gtk_widget_set_sensitive
     (info->pref, xine_post_input (info->plugin, "parameters") != NULL);
+
+  gtk_tooltips_set_tip (tips, (GtkWidget *)menu,
+			xine_get_post_plugin_description
+			  ? xine_get_post_plugin_description (xine, name)
+			  : NULL,
+			NULL);
 }
 
 static void
@@ -797,7 +824,8 @@ post_config_post_chain_move_plugin (GtkButton *widget, gxine_post_t *info)
   if (--pos < 0)
     return;
   logprintf ("post_config: moving plugin '%s' up\n", info->name);
-  g_slist_remove (info->parent->plugins, info); /* this is not the first item */
+  /* we're not removing the first item, but keep the compiler happy :-) */
+  info->parent->plugins = g_slist_remove (info->parent->plugins, info);
   info->parent->plugins = g_slist_insert (info->parent->plugins, info, pos);
   gtk_box_reorder_child (GTK_BOX(info->box->parent), info->box, pos);
   gtk_widget_set_sensitive (info->up, pos);
@@ -806,8 +834,6 @@ post_config_post_chain_move_plugin (GtkButton *widget, gxine_post_t *info)
      TRUE);
 }
 
-extern const char *xine_get_post_plugin_description () __attribute__((weak));
-
 static void
 post_config_post_add_plugin (gxine_chain_t *chain, xine_post_t *post,
 			     gboolean enabled, char *name, char *params)
@@ -815,6 +841,7 @@ post_config_post_add_plugin (gxine_chain_t *chain, xine_post_t *post,
   gxine_post_t *info = malloc (sizeof (gxine_post_t));
   GtkWidget *w, *m;
   int i, history = 0;
+  const char *hist_desc = NULL;
 
   logprintf ("post_config: adding plugin '%s'\n", name);
   info->parent = chain;
@@ -836,7 +863,10 @@ post_config_post_add_plugin (gxine_chain_t *chain, xine_post_t *post,
 		     : NULL;
     GtkWidget *item;
     if (!strcmp (chain->names[i], name))
+    {
       history = i;
+      hist_desc = desc;
+    }
     gtk_menu_shell_append (GTK_MENU_SHELL(m),
 			item =   gtk_menu_item_new_with_label (chain->names[i]));
     if (desc)
@@ -854,6 +884,8 @@ post_config_post_add_plugin (gxine_chain_t *chain, xine_post_t *post,
   gtk_box_pack_start_defaults (GTK_BOX(info->box), w);
   g_signal_connect (G_OBJECT(w), "changed",
 		    G_CALLBACK(post_config_post_chain_set_plugin), info);
+  if (hist_desc)
+    gtk_tooltips_set_tip (tips, w, hist_desc, NULL);
 
   info->up = w = ui_button_new_stock (GTK_STOCK_GO_UP);
   gtk_widget_set_sensitive (w, !!chain->plugins); /* disable for first plugin */
@@ -890,7 +922,7 @@ post_config_post_chain_add (GtkButton *button, gxine_chain_t *chain)
 				 strdup (""));
   else
     display_error_modal (FROM_XINE, _("Plugin error"),
-			 _("Couldn't initialise plugin '%s' for configuration"),
+			 _("Couldn't initialise plugin ‘%s’ for configuration"),
 			 chain->names[0]);
 }
 
@@ -1020,6 +1052,7 @@ post_config_post_chain_window (gxine_chain_t *chain)
   hide_on_delete (chain->dbox, NULL);
   g_signal_connect (G_OBJECT(chain->dbox), "response",
 		    G_CALLBACK(post_config_post_chain_response), chain);
+  ui_add_undo_response (chain->dbox, NULL);
   if (!tips)
     tips = gtk_tooltips_new ();
 
