@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2005 the xine project
+ * Copyright (C) 2001-2006 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: preferences.c,v 1.54 2005/11/10 18:55:29 dsalt Exp $
+ * $Id: preferences.c,v 1.61 2006/04/05 19:08:33 dsalt Exp $
  *
  * Functions to stup and deal with a preferences dialog interfacing with
  * the xine config system.
@@ -40,7 +40,6 @@
 #include "preferences.h"
 #include "utils.h"
 
-#define XP_UNIX
 #include <jsapi.h>
 
 
@@ -67,6 +66,7 @@ typedef union {
 
 static GtkWidget  *prefs_dialog;
 static GNode	  *prefs = NULL;
+static GData	  *prefs_map = NULL;
 static int         is_visible;
 static GtkTooltips *tooltips = NULL;
 static int         update_lock = 0;
@@ -79,7 +79,7 @@ static int select_show_prefs_internal (GNode *page, int exp)
   pref_t *parent = page->parent ? page->parent->data : NULL;
   GNode *child;
 
-  if (parent && parent->notebook == (void *) 1)
+  if (parent && parent->notebook == GINT_TO_POINTER (1))
     return 0; /* called too early */
 
   child = g_node_first_child (page);
@@ -127,14 +127,6 @@ static int select_show_prefs_internal (GNode *page, int exp)
     gtk_notebook_append_page (GTK_NOTEBOOK(parent->notebook), item->page.page,
 			      gtk_label_new (item->page.label));
 
-  if (parent)
-  {
-    gboolean multi =
-      gtk_notebook_get_n_pages (GTK_NOTEBOOK(parent->notebook)) > 1;
-    gtk_notebook_set_show_tabs (GTK_NOTEBOOK(parent->notebook), multi);
-    gtk_notebook_set_show_border (GTK_NOTEBOOK(parent->notebook), multi);
-  }
-
   return state;
 }
 
@@ -143,8 +135,7 @@ static gboolean select_show_pref_widgets (void *data)
   static const int experience_values[] = { 0, 10, 20, 30 };
   int exp = *(int *)data;
 
-  if (exp >= 0 && exp < sizeof (experience_values) /
-			sizeof (experience_values[0]))
+  if (exp >= 0 && exp < (int) G_N_ELEMENTS (experience_values))
     exp = experience_values[exp];
   else
     exp = 0;
@@ -300,14 +291,19 @@ static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
 {
   GtkWidget *widget;
 
-  switch (entry->type) {
+  switch (entry->type)
+  {
   case XINE_CONFIG_TYPE_ENUM:
     {
       int i;
       widget = gtk_combo_box_new_text ();
       for (i = 0; entry->enum_values[i]; ++i)
-	gtk_combo_box_append_text (GTK_COMBO_BOX(widget),
-				   entry->enum_values[i]);
+      {
+	const char *label = gettext (entry->enum_values[i]);
+	if (label == entry->enum_values[i])
+	  label = dgettext (LIB_PACKAGE, label);
+	gtk_combo_box_append_text (GTK_COMBO_BOX(widget), label);
+      }
       gtk_combo_box_set_active (GTK_COMBO_BOX(widget), entry->num_value);
       g_signal_connect (G_OBJECT(widget), "changed",
 			G_CALLBACK(enum_cb), strdup(entry->key));
@@ -361,9 +357,10 @@ static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
 
   default:
     widget = gtk_label_new ("?");
-    printf ("preferences: unknown type for entry '%s'\n", entry->key);
+    g_print (_("preferences: unknown type for entry ‘%s’\n"), entry->key);
   }
 
+  g_datalist_set_data (&prefs_map, entry->key, widget);
   gtk_tooltips_set_tip (tooltips, widget, entry->help, NULL);
   return widget;
 }
@@ -406,10 +403,10 @@ static GNode *create_item (const xine_cfg_entry_t *entry)
 static GNode *create_page (const char *key, size_t length)
 {
   pref_page_t *page = malloc (sizeof (pref_page_t));
-  page->notebook = (void *) 1; /* dummy value - filled in later */
+  page->notebook = GINT_TO_POINTER (1); /* dummy value - filled in later */
   page->page = page->table = NULL;
   page->prefix = malloc (length + 1);
-  sprintf (page->prefix, "%.*s", length, key);
+  sprintf (page->prefix, "%.*s", (int) length, key);
   page->label = strrchr (page->prefix, '.');
   if (page->label)
     ++page->label;
@@ -499,6 +496,29 @@ static gboolean unmix_mixed_page (GNode *node, gboolean *done)
   return FALSE;
 }
 
+static gboolean focus_item_cb (GtkWidget *widget, GtkDirectionType *dir,
+			       GNode *node)
+{
+  const pref_t *pref = node->data;
+  const pref_t *parent = node->parent ? node->parent->data : NULL;
+  GtkWidget *viewport = parent->page.table->parent;
+  GtkAdjustment *adj = gtk_viewport_get_vadjustment ((GtkViewport *)viewport);
+
+  int wt = pref->item.editable->allocation.y;
+  int wb = wt + pref->item.editable->allocation.height;
+  int lt = pref->item.label->allocation.y;
+  int lb = lt + pref->item.label->allocation.height;
+  int top = wt < lt ? wt : lt;
+  int bottom = wb > lb ? wb : lb;
+
+  if (top < adj->value)
+    gtk_adjustment_set_value (adj, top);
+  else if (bottom + 4 > adj->value + viewport->allocation.height)
+    gtk_adjustment_set_value (adj, bottom - viewport->allocation.height + 4);
+  
+  return FALSE;
+}
+
 static gboolean put_content (GNode *node, gpointer data)
 {
   pref_t *pref = node->data;
@@ -549,6 +569,8 @@ static gboolean put_content (GNode *node, gpointer data)
 		      GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
     gtk_table_attach (table, pref->item.separator, 0, 4, r + 1, r + 2,
 		      GTK_EXPAND | GTK_FILL, GTK_SHRINK | GTK_FILL, 5, 5);
+    g_signal_connect (pref->item.editable, "focus",
+		      focus_item_cb, node);
   }
 
   return FALSE;
@@ -565,7 +587,8 @@ static GtkWidget *make_prefs_window (void)
   tooltips = gtk_tooltips_new ();
   prefs = create_page ("", 0); /* root page */
 
-  do {
+  do
+  {
     const char *point;
     GNode *section;
 
@@ -615,10 +638,8 @@ static void response_cb (GtkDialog *dbox, int response, gpointer data)
   gchar *fname;
   switch (response)
   {
-  case GTK_RESPONSE_REJECT:
-    
   case GTK_RESPONSE_ACCEPT:
-    fname = g_strconcat(g_get_home_dir(), "/.gxine/config", NULL);
+    fname = get_config_filename (FILE_CONFIG);
     xine_config_save (xine, fname);
     g_free (fname);
     break;
@@ -639,10 +660,13 @@ static JSBool js_preferences_show (JSContext *cx, JSObject *obj, uintN argc,
 
 void preferences_show (void)
 {
-  if (is_visible) {
+  if (is_visible)
+  {
     is_visible = FALSE;
     gtk_widget_hide (prefs_dialog);
-  } else {
+  }
+  else
+  {
     is_visible = TRUE;
     window_show (prefs_dialog, NULL);
   }
@@ -688,3 +712,36 @@ void preferences_init (void)
   se_defun (gse, NULL, "preferences_show", js_preferences_show, 0, 0,
 	    SE_GROUP_DIALOGUE, NULL, NULL);
 }
+
+void preferences_update_entry (const xine_cfg_entry_t *entry)
+{
+  gpointer widget = g_datalist_get_data (&prefs_map, entry->key);
+
+  if (widget)
+    switch (entry->type)
+    {
+    case XINE_CONFIG_TYPE_ENUM:
+      gtk_combo_box_set_active (widget, entry->num_value);
+      break;
+
+    case XINE_CONFIG_TYPE_STRING:
+      gtk_entry_set_text (widget, entry->str_value);
+      break;
+
+    case XINE_CONFIG_TYPE_RANGE: /* slider */
+      gtk_range_set_value (widget, entry->num_value);
+      break;
+
+    case XINE_CONFIG_TYPE_NUM:
+      gtk_spin_button_set_value (widget, entry->num_value);
+      break;
+
+    case XINE_CONFIG_TYPE_BOOL:
+      gtk_toggle_button_set_active (widget, entry->num_value == 1);
+      break;
+
+    default:;
+    }
+
+  xine_config_update_entry (xine, entry);
+}
