@@ -1,4 +1,4 @@
-/* 
+/*
  * Copyright (C) 2003-2005 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
@@ -13,15 +13,15 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: script_engine.h,v 1.24 2005/11/11 20:52:10 dsalt Exp $
+ * $Id: script_engine.h,v 1.31 2006-03-25 20:16:52 dsalt Exp $
  *
  * gate to the spider monkey javascript interpreter
  *
  * provides functions to generate interpreter instances
  * customized for controlling the xine engine and the gxine
- * frontend. 
+ * frontend.
  *
  */
 
@@ -30,8 +30,11 @@
 
 #include <glib.h>
 
-#define XP_UNIX
+#ifndef XP_UNIX
+# define XP_UNIX
+#endif
 #include <jsapi.h>
+#include <pthread.h>
 
 typedef int (*se_prop_int_cb_t) (void *user_data, char *property, int *num);
 typedef void (*se_print_cb_t) (void *user_data, const char *str, ...)
@@ -58,7 +61,8 @@ struct se_s {
   JSContext     *cx;
   JSObject      *global;
   se_o_t        *g;
-  
+  pthread_mutex_t lock;
+
   jsval          rval;
   JSString      *str;
 
@@ -82,7 +86,7 @@ typedef enum {
   SE_TYPE_INT,
   SE_TYPE_DOUBLE,
 } se_prop_type_t;
- 
+
 typedef union {
   char *s;
   int i;
@@ -103,16 +107,16 @@ struct se_prop_s {
   gboolean	     constant;
 };
 typedef struct se_prop_s se_prop_t;
- 
+
 typedef int (*se_prop_cb_t) (void *, se_t *, se_o_t *, se_prop_t *, se_prop_read_t r);
- 
+
 typedef struct {
- 
+
   se_prop_cb_t   cb;
   void          *user_data;
- 
+
 } se_prop_listener_t;
- 
+
 struct se_o_s {
 
   JSObject          *obj;
@@ -123,7 +127,7 @@ struct se_o_s {
   GList             *children;
 
   GList             *functions; /* methods */
- 
+
   GList             *properties;
 
   void              *user_data;
@@ -132,8 +136,10 @@ struct se_o_s {
   const char	    *help;
 };
 
+extern se_o_t *event_obj;
+
 /*
- * create a javascript engine with predefined xine functions 
+ * create a javascript engine with predefined xine functions
  * and objects
  */
 se_t *se_new (void);
@@ -147,7 +153,7 @@ int se_eval (se_t *se, const gchar *script, se_o_t *obj,
 int se_eval_ext (se_t *, const gchar *, se_o_t *, se_print_cb_t, void *,
 		 se_error_cb_t, const char *);
 
-gchar *se_result_str (se_t *se) ;
+gchar *se_result_str (se_t *se);
 
 int se_result_int (se_t *se, JSInt32 *num);
 int se_result_double (se_t *se, JSFloat64 *num);
@@ -164,6 +170,8 @@ se_o_t *se_create_object (se_t *se, se_o_t *parent /* may be NULL */,
                           se_group_t group, const char *help);
 
 se_o_t *se_find_object (se_t *se, se_o_t *parent, const char *name);
+se_o_t *se_get_object (se_t *se, se_o_t *parent, const char *name);
+  /* never returns NULL, but don't use if the returned object may be modified */
 
 /*
  * create functions (methods)
@@ -240,6 +248,12 @@ JSBool se_warn_initialisation (JSContext *, const char *);
 #define se_log(FMT, ...)
 #endif
 
+#define se_log_fncall_deprecated(func) \
+  do { \
+    se_log_fncall ((func)); \
+    JS_ReportWarning (cx, _("Deprecated: %s"), func); \
+  } while (0)
+
 #define se_log_fncall_checkinit(func) \
   do \
   { \
@@ -248,55 +262,62 @@ JSBool se_warn_initialisation (JSContext *, const char *);
       return se_warn_initialisation (cx, _func_); \
     se_log_fncall (_func_); \
   } while (0)
-    
+
 
 /* Validation - assumes se_t *se, uintN argc, jsval *argv */
 
 #define se_argc_check(value, FUNC) \
   if (argc != (value)) { \
-    se->print_cb (se->print_cb_data, \
+    JS_ReportError (cx, \
 		  _("error: %s() needs %d parameters\n"), (FUNC), (value)); \
     return JS_TRUE; \
   }
 
 #define se_argc_check_range(min, max, FUNC) \
   if (argc < (uintN)(min) || argc > (uintN)(max)) { \
-    se->print_cb (se->print_cb_data, \
+    JS_ReportError (cx, \
 	   _("error: %s() needs %d...%d parameters\n"), (FUNC), (min), (max)); \
     return JS_TRUE; \
   }
 
 #define se_argc_check_max(max, FUNC) \
   if (argc > (uintN)(max)) { \
-    se->print_cb (se->print_cb_data, \
+    JS_ReportError (cx, \
 		  _("error: %s() needs 0...%d parameters\n"), (FUNC), (max)); \
     return JS_TRUE; \
   }
 
 #define se_arg_is_object(n, FUNC) \
   if (!JSVAL_IS_OBJECT (argv[(n)])) { \
-    se->print_cb (se->print_cb_data, \
+    JS_ReportError (cx, \
 		 _("error: %s() argument %d is not an object\n"), (FUNC), (n)+1); \
     return JS_TRUE; \
   }
 
 #define se_arg_is_int(n, FUNC) \
   if (!JSVAL_IS_INT (argv[(n)])) { \
-    se->print_cb (se->print_cb_data, \
+    JS_ReportError (cx, \
 		 _("error: %s() argument %d is not an int\n"), (FUNC), (n)+1); \
     return JS_TRUE; \
   }
 
 #define se_arg_is_int_or_bool(n, FUNC) \
   if (!JSVAL_IS_INT (argv[(n)]) && !JSVAL_IS_BOOLEAN (argv[(n)])) { \
-    se->print_cb (se->print_cb_data, \
+    JS_ReportError (cx, \
 		 _("error: %s() argument %d is not an int or a boolean\n"), (FUNC), (n)+1); \
     return JS_TRUE; \
   }
 
 #define se_arg_is_string(n, FUNC) \
   if (!JSVAL_IS_STRING (argv[(n)])) { \
-    se->print_cb (se->print_cb_data, \
+    JS_ReportError (cx, \
+	       _("error: %s() argument %d is not a string\n"), (FUNC), (n)+1); \
+    return JS_TRUE; \
+  }
+
+#define se_arg_is_string_or_null(n, FUNC) \
+  if (!JSVAL_IS_STRING (argv[(n)]) && !JSVAL_IS_NULL (argv[(n)])) { \
+    JS_ReportError (cx, \
 	       _("error: %s() argument %d is not a string\n"), (FUNC), (n)+1); \
     return JS_TRUE; \
   }
