@@ -68,7 +68,7 @@ static int make_socket (char *path)
 
   adr.u.sun_family = AF_UNIX;
   strncpy (adr.u.sun_path, path, sizeof (adr.u.sun_path)-1);
-  
+
   /* bind the name to the descriptor */
   if (bind (sock, &adr.s, sizeof (adr)) < 0)
   {
@@ -91,7 +91,8 @@ static void sock_write (int fd, char *msg)
     int n = write (fd, msg+pos, len-pos);
 
     if (n <= 0)
-      switch (errno) {
+      switch (errno)
+      {
       case EAGAIN:
 	continue;
       default:
@@ -185,7 +186,7 @@ static __attribute__ ((noreturn)) void *socket_listener (void *data)
 void server_setup (void)
 {
   char filename[FILENAME_MAX];
-  snprintf (filename, sizeof (filename), SOCKET_FILENAME, getenv ("HOME")); 
+  snprintf (filename, sizeof (filename), SOCKET_FILENAME, getenv ("HOME"));
   gxsocket = make_socket (filename);
 }
 
@@ -242,7 +243,7 @@ int server_client_connect (void)
   signal (SIGPIPE, sigpipe_handler);
 
   /* server filename */
-  snprintf (filename, PATH_MAX, SOCKET_FILENAME, getenv ("HOME")); 
+  snprintf (filename, PATH_MAX, SOCKET_FILENAME, getenv ("HOME"));
 
   logprintf ("server: trying to connect to already running instance of gxine (%s)...\n",
 	    filename);
@@ -251,7 +252,7 @@ int server_client_connect (void)
 
   /* initialize the client address structure */
   cli_adr.u.sun_family = AF_UNIX;
- 
+
   sprintf (tstr, "/tmp/gxine_%d", getpid ());
   strcpy (cli_adr.u.sun_path, tstr);
   unlink (tstr);
@@ -269,13 +270,11 @@ int server_client_connect (void)
 
   if (connect (client_fd, &serv_adr.s, sizeof (serv_adr)) < 0)
   {
-#ifdef LOG
-    perror ("connect");
-#endif
+    logperror ("connect");
     server_client_stop ();
     return 0;
   }
-  
+
   g_print (_("server: connected to existing gxine instance.\n"));
   atexit (server_client_stop);
 
