@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2004-2005 the xine-project
+ * Copyright (C) 2004-2006 the xine-project
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
@@ -13,10 +13,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: ui.c,v 1.33 2005/11/29 21:49:24 dsalt Exp $
+ * $Id: ui.c,v 1.50 2006/04/08 21:34:50 dsalt Exp $
  *
  * various UI bits:
  * - button registry and update (state display etc.)
@@ -29,6 +29,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include <pthread.h>
 
 #include "ui.h"
@@ -36,6 +37,7 @@
 #include "engine.h"
 #include "player.h"
 #include "playlist.h"
+#include "preferences.h"
 #include "gtkvideo.h"
 
 /*
@@ -50,6 +52,8 @@ gboolean fs_toolbar_visible = FALSE;
 gboolean wm_toolbar_visible = TRUE;
 gboolean wm_toolbar_snap = TRUE;
 
+GtkIconSize icon_size_logo;
+
 static ui_status_t status = UI_CURRENT_STATE;
 pthread_mutex_t status_lock = PTHREAD_MUTEX_INITIALIZER;
 
@@ -58,6 +62,13 @@ static GtkObject *c_adjustments[Control_Adjustments] = { NULL };
 
 /* (Internal) Reflect play/pause/stop status in control buttons */
 
+static void ui_set_list (int index, const char *property, gboolean state)
+{
+  GSList *btn;
+  foreach_glist (btn, c_buttons[index])
+    g_object_set (G_OBJECT (btn->data), property, state, NULL);
+}
+
 static void ui_int_set_status (ui_status_t status)
 {
   char states[Control_PlayerButtons] = { 0 };
@@ -78,12 +89,11 @@ static void ui_int_set_status (ui_status_t status)
   no_recursion = TRUE;
   for (i = 0; i < Control_PlayerButtons; ++i)
   {
-    GSList *btn;
-    foreach_glist (btn, c_buttons[i])
-      g_object_set (G_OBJECT(btn->data), "active", states[i], NULL);
-    if (i == Control_PAUSE || i == Control_REWIND || i == Control_FASTFWD)
-      foreach_glist (btn, c_buttons[i])
-	g_object_set (G_OBJECT(btn->data), "sensitive", !live, NULL);
+    ui_set_list (i, "active", states[i]);
+    if (i == Control_PLAY)
+      ui_set_list (i, "sensitive", !!playlist_size ());
+    else if (i == Control_PAUSE || i == Control_REWIND || i == Control_FASTFWD)
+      ui_set_list (i, "sensitive", !live);
   }
   no_recursion = FALSE;
 }
@@ -136,7 +146,7 @@ void ui_set_status (ui_status_t newstatus)
 
 void ui_register_control_button (control_button_t item, GtkWidget *widget)
 {
-  g_return_if_fail (item >= 0 && item < Control_Buttons);
+  g_return_if_fail (item < Control_Buttons);
   c_buttons[item] = g_slist_append (c_buttons[item], widget);
 }
 
@@ -170,9 +180,9 @@ static const ui_adjustable_t ranges[] = {
   [Control_EQ_8K]		= {   100,	 0,    100,    1,    10, 0, "gui.eq_8K",		 "eq_8K",	  XINE_PARAM_EQ_8000HZ },
   [Control_EQ_16K]		= {   100,	 0,    100,    1,    10, 0, "gui.eq_16K",		 "eq_16K",	  XINE_PARAM_EQ_16000HZ },
 };
-static gdouble starts[sizeof (ranges) / sizeof (ranges[0])];
-static gdouble inits[sizeof (ranges) / sizeof (ranges[0])];
-static se_o_t *jsobjs[sizeof (ranges) / sizeof (ranges[0])];
+static gdouble starts[G_N_ELEMENTS (ranges)];
+static gdouble inits[G_N_ELEMENTS (ranges)];
+static se_o_t *jsobjs[G_N_ELEMENTS (ranges)];
 
 /* true if not volume setting or audio.remember_volume is set */
 static int check_remember_volume (const ui_adjustable_t *info)
@@ -200,7 +210,7 @@ static void ui_adjustment_value_changed_cb (GtkAdjustment *adj,
       xine_config_lookup_entry (xine, info->setting, &entry))
   {
     entry.num_value = adj->value;
-    xine_config_update_entry (xine, &entry);
+    preferences_update_entry (&entry);
   }
 }
 
@@ -221,16 +231,20 @@ ui_seeker_value_changed_cb (GtkAdjustment *seeker, gpointer data)
 
 GtkObject *ui_register_control_adjustment (control_adjustment_t item)
 {
-  g_return_val_if_fail (item >= 0 && item < Control_Adjustments, NULL);
+  g_return_val_if_fail (item < Control_Adjustments, NULL);
 
   if (!c_adjustments[item])
   {
     xine_cfg_entry_t entry;
     int start = ranges[item].start;
 
-    if (ranges[item].setting && check_remember_volume (&ranges[item]) &&
-	xine_config_lookup_entry (xine, ranges[item].setting, &entry))
-      start = entry.num_value;
+    if (ranges[item].setting)
+    {
+      if (!check_remember_volume (&ranges[item]))
+        start = xine_get_param (stream, XINE_PARAM_AUDIO_VOLUME);
+      else if (xine_config_lookup_entry (xine, ranges[item].setting, &entry))
+	start = entry.num_value;
+    }
 
     starts[item] = start;
 
@@ -270,7 +284,7 @@ void ui_set_control_button (control_button_t item, gboolean state)
 
 void ui_set_control_adjustment (control_adjustment_t item, gdouble value)
 {
-  g_return_if_fail (item >= 0 && item < Control_Adjustments);
+  g_return_if_fail (item < Control_Adjustments);
   no_recursion = TRUE;
   if ((GTK_ADJUSTMENT(c_adjustments[item]))->value != value)
     g_object_set (G_OBJECT(c_adjustments[item]), "value", value, NULL);
@@ -279,20 +293,20 @@ void ui_set_control_adjustment (control_adjustment_t item, gdouble value)
 
 void ui_reset_control_adjustment (control_adjustment_t item)
 {
-  g_return_if_fail (item >= 0 && item < Control_Adjustments);
+  g_return_if_fail (item < Control_Adjustments);
   g_object_set (G_OBJECT(c_adjustments[item]), "value",
 		ranges[item].start, NULL);
 }
 
 void ui_revert_control_adjustment (control_adjustment_t item)
 {
-  g_return_if_fail (item >= 0 && item < Control_Adjustments);
+  g_return_if_fail (item < Control_Adjustments);
   g_object_set (G_OBJECT(c_adjustments[item]), "value", starts[item], NULL);
 }
 
 void ui_clear_control_adjustment (control_adjustment_t item)
 {
-  g_return_if_fail (item >= 0 && item < Control_Adjustments);
+  g_return_if_fail (item < Control_Adjustments);
   if (check_remember_volume (&ranges[item]))
     xine_set_param (stream, ranges[item].param, inits[item]);
 }
@@ -301,7 +315,7 @@ void ui_clear_control_adjustment (control_adjustment_t item)
 
 void ui_xine_set_param_from_adjustment (control_adjustment_t item)
 {
-  g_return_if_fail (item >= 0 && item < Control_Adjustments);
+  g_return_if_fail (item < Control_Adjustments);
   gtk_adjustment_value_changed (GTK_ADJUSTMENT(c_adjustments[item]));
 }
 
@@ -342,6 +356,41 @@ GtkWidget *ui_button_new_stock_mnemonic (const char *stock,
   return button;
 }
 
+static GtkWidget *ui_container_new_icon_name (GtkWidget *box, const char *name)
+{
+  GtkWidget *img = gtk_image_new_from_icon_name (name, GTK_ICON_SIZE_BUTTON);
+  gtk_misc_set_padding (GTK_MISC(img), 0, 0);
+  if (box)
+    gtk_container_add (GTK_CONTAINER(box), img);
+  return box ? box : img;
+}
+
+GtkWidget *ui_button_new_icon_name (const char *name)
+{
+  return ui_container_new_icon_name (gtk_button_new (), name);
+}
+
+GtkWidget *ui_toggle_button_new_icon_name (const char *name)
+{
+  return ui_container_new_icon_name (gtk_toggle_button_new (), name);
+}
+
+GtkWidget *ui_button_new_icon_name_mnemonic (const char *name,
+					     const char *mnemonic)
+{
+  GtkWidget *button = gtk_button_new ();
+  GtkWidget *align = gtk_alignment_new (0.5, 0.5, 0, 0);
+  GtkWidget *hbox = gtk_hbox_new (FALSE, 2);
+  /* hbox in alignment in button => hbox is not expanded to fill the button */
+  gtk_container_add (GTK_CONTAINER(align), hbox);
+  gtk_container_add (GTK_CONTAINER(button), align);
+  gtk_box_pack_start (GTK_BOX(hbox), ui_container_new_icon_name (NULL, name),
+		      FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX(hbox), gtk_label_new_with_mnemonic (mnemonic),
+		      FALSE, FALSE, 0);
+  return button;
+}
+
 /* Menu/toolbar basics */
 
 GtkUIManager *ui_create_manager (const char *label, GtkWidget *window)
@@ -404,16 +453,15 @@ static void set_fs_toolbar_top (int v)
   else
     gtk_action_activate (action_items.fs_toolbar_pos->data); /* bottom */
 }
-            
+
 #ifdef WITH_DEPRECATED
 
-static JSBool js_fs_toolbar_show (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_fs_toolbar_show (JSContext *cx, JSObject *obj, uintN argc,
 				  jsval *argv, jsval *rval)
 {
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
   JSBool show;
 
-  se_log_fncall ("toolbar_show");
+  se_log_fncall_deprecated ("toolbar_show");
   se_argc_check_max (1, "toolbar_show");
 
   if (argc == 1)
@@ -424,21 +472,17 @@ static JSBool js_fs_toolbar_show (JSContext *cx, JSObject *obj, uintN argc,
   else
     show = !fs_toolbar_visible;
 
-  if (engine_mutex_trylock (se))
-    return JS_TRUE;
   window_fs_toolbar_show (show);
-  pthread_mutex_unlock (&engine_lock);
 
   return JS_TRUE;
 }
 
-static JSBool js_fs_toolbar_position (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_fs_toolbar_position (JSContext *cx, JSObject *obj, uintN argc,
 				      jsval *argv, jsval *rval)
 {
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
   JSBool top;
 
-  se_log_fncall ("set_toolbar_position");
+  se_log_fncall_deprecated ("set_toolbar_position");
   se_argc_check_max (1, "set_toolbar_position");
 
   if (argc == 1)
@@ -449,11 +493,8 @@ static JSBool js_fs_toolbar_position (JSContext *cx, JSObject *obj, uintN argc,
   else
     top = !fs_toolbar_at_top;
 
-  if (engine_mutex_trylock (se))
-    return JS_TRUE;
   set_fs_toolbar_top
     (gtk_radio_action_get_current_value (action_items.fs_toolbar_pos->data));
-  pthread_mutex_unlock (&engine_lock);
 
   return JS_TRUE;
 }
@@ -465,7 +506,7 @@ static JSBool js_fs_toolbar_position (JSContext *cx, JSObject *obj, uintN argc,
 static int ui_lookup_js_obj (const JSObject *obj)
 {
   int i;
-  for (i = 0; i < sizeof (ranges) / sizeof (ranges[0]); ++i)
+  for (i = 0; i < (int) G_N_ELEMENTS (ranges); ++i)
     if (jsobjs[i] && jsobjs[i]->obj == obj)
       return i;
   abort (); /* can't happen */
@@ -474,32 +515,20 @@ static int ui_lookup_js_obj (const JSObject *obj)
 static JSBool js_control_revert (JSContext *cx, JSObject *obj, uintN argc,
 				 jsval *argv, jsval *rval)
 {
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  *rval = JSVAL_FALSE;
-
   se_log_fncall ("<control>.revert");
   se_argc_check (0, "<control>.revert");
-  if (engine_mutex_trylock (se))
-    return JS_TRUE;
   ui_revert_control_adjustment (ui_lookup_js_obj (obj));
-  pthread_mutex_unlock (&engine_lock);
-
+  *rval = JSVAL_VOID;
   return JS_TRUE;
 }
 
 static JSBool js_control_reset (JSContext *cx, JSObject *obj, uintN argc,
 				jsval *argv, jsval *rval)
 {
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  *rval = JSVAL_FALSE;
-
   se_log_fncall ("<control>.reset");
   se_argc_check (0, "<control>.reset");
-  if (engine_mutex_trylock (se))
-    return JS_TRUE;
   ui_clear_control_adjustment (ui_lookup_js_obj (obj));
-  pthread_mutex_unlock (&engine_lock);
-
+  *rval = JSVAL_VOID;
   return JS_TRUE;
 }
 
@@ -526,16 +555,11 @@ static int ui_prop_set_int (void *data, se_t *se, se_o_t *o,
 			    se_prop_t *se_prop, se_prop_read_t value)
 {
   const ui_property_t *prop = data;
-
-  if (!engine_mutex_trylock (se))
-  {
-    value.i = ui_property_clip_int (prop, value.i);
-    if (prop->set)
-      prop->set (value.i);
-    else
-      xine_set_param (stream, prop->param, value.i);
-    pthread_mutex_unlock (&engine_lock);
-  }
+  value.i = ui_property_clip_int (prop, value.i);
+  if (prop->set)
+    prop->set (value.i);
+  else
+    xine_set_param (stream, prop->param, value.i);
   return 0;
 }
 
@@ -543,14 +567,10 @@ static int ui_prop_set_bool (void *data, se_t *se, se_o_t *o,
 			     se_prop_t *se_prop, se_prop_read_t value)
 {
   const ui_property_t *prop = data;
-  if (!engine_mutex_trylock (se))
-  {
-    if (prop->set)
-      prop->set (!!value.i);
-    else
-      xine_set_param (stream, prop->param, !!value.i);
-    pthread_mutex_unlock (&engine_lock);
-  }
+  if (prop->set)
+    prop->set (!!value.i);
+  else
+    xine_set_param (stream, prop->param, !!value.i);
   return 0;
 }
 
@@ -565,18 +585,18 @@ static JSBool ui_prop_toggle_bool (JSContext *cx, JSObject *obj,
   return JS_TRUE;
 }
 
-void ui_create_properties (const ui_property_t *prop, se_o_t *parent, 
-			   se_prop_type_t type, se_prop_cb_t cb)
+void ui_create_properties (const ui_property_t *prop, se_o_t *parent,
+			   se_prop_type_t type)
 {
   int i;
-  
-  if (!cb)
-    switch (type)
-    {
-    case SE_TYPE_INT:  cb = ui_prop_set_int;  break;
-    case SE_TYPE_BOOL: cb = ui_prop_set_bool; break;
-    default:;
-    }
+  se_prop_cb_t cb = NULL;
+
+  switch (type)
+  {
+  case SE_TYPE_INT: cb = ui_prop_set_int; break;
+  case SE_TYPE_BOOL: cb = ui_prop_set_bool; break;
+  default:;
+  }
 
   for (i = 0; prop[i].name; ++i)
   {
@@ -590,8 +610,13 @@ void ui_create_properties (const ui_property_t *prop, se_o_t *parent,
       se_prop_create (gse, obj, "v", "", NULL, 0, type, FALSE);
       se_prop_set_reader_func (gse, obj, "v", prop[i].get);
     }
-    if (cb)
+
+    if (prop[i].listen.cb)
+      se_prop_add_listener (gse, obj, "v", prop[i].listen.cb,
+			    prop[i].listen.user_data);
+    else
       se_prop_add_listener (gse, obj, "v", cb, &prop[i]);
+
     switch (type)
     {
     case SE_TYPE_INT:
@@ -612,19 +637,19 @@ void ui_create_properties (const ui_property_t *prop, se_o_t *parent,
 static void post_deinterlace_plugin_cb (void *data, xine_cfg_entry_t *cfg)
 {
   if (gtv)
-    gtk_video_set_post_plugins_deinterlace (GTK_VIDEO(gtv), cfg->str_value);
+    gtk_video_set_post_plugins_deinterlace ((GtkVideo *)gtv, cfg->str_value);
 }
 
 static void post_plugins_video_cb (void *data, xine_cfg_entry_t *cfg)
 {
   if (gtv)
-    gtk_video_set_post_plugins_video (GTK_VIDEO(gtv), cfg->str_value);
+    gtk_video_set_post_plugins_video ((GtkVideo *)gtv, cfg->str_value);
 }
 
 static void post_plugins_audio_cb (void *data, xine_cfg_entry_t *cfg)
 {
   if (gtv)
-    gtk_video_set_post_plugins_audio (GTK_VIDEO(gtv), cfg->str_value,
+    gtk_video_set_post_plugins_audio ((GtkVideo *)gtv, cfg->str_value,
 				      audio_port);
 }
 
@@ -642,7 +667,7 @@ void ui_preferences_register (xine_t *this)
     NULL
   };
   const char **vis_labels, *const *vis_src;
-  int i;
+  unsigned int i;
 
   /* Register the experience level setting */
   xine_config_register_enum
@@ -652,7 +677,7 @@ void ui_preferences_register (xine_t *this)
      0, NULL, CONFIG_DATA_NONE);
 
   /* Register a few audio/video config items */
-  for (i = 0; i < sizeof (ranges) / sizeof (ranges[0]); ++i)
+  for (i = 0; i < G_N_ELEMENTS (ranges); ++i)
   {
     if (ranges[i].setting && !strncmp (ranges[i].setting, "gui.", 4))
       xine_config_register_range (this, ranges[i].setting, ranges[i].start,
@@ -709,6 +734,11 @@ void ui_preferences_register (xine_t *this)
      _("Post-plugin to be used when playing streams without video"),
      30, NULL, NULL);
 
+  xine_config_register_bool
+    (this, "gui.post_plugins.always_show_logo", 1,
+     _("In windowed mode, whether the logo is shown after playing an audio-only stream if no audio visualisation was active."), NULL,
+     0, NULL, NULL);
+
   xine_config_register_enum
     (this, "gui.fullscreen_toolbar", 1, tbar_pos_labels,
      _("Default position & visibility of the full-screen toolbar"),
@@ -719,6 +749,11 @@ void ui_preferences_register (xine_t *this)
      _("In windowed mode, whether the toolbar is in a separate window"), NULL,
      0, NULL, NULL);
 
+  xine_config_register_bool
+    (this, "gui.show_splash", 1,
+     _("Display splash screen"),
+     _("If enabled, gxine will display its splash screen"),
+     0, NULL, NULL);
 }
 
 static int ui_listener (void *data, se_t *se, se_o_t *o, se_prop_t *prop,
@@ -739,16 +774,16 @@ static GtkIconSource *make_icon_source (const char *name, GtkStateType state,
 {
   GtkIconSource *isrc = gtk_icon_source_new ();
   char *filename = *name == '/'
-		 ? g_strconcat (icondir, name, NULL)
-		 : g_strconcat (pixmapdir, "/", name, NULL);
+		   ? g_build_filename (icondir, name, NULL)
+		   : g_build_filename (pixmapdir, name, NULL);
   gtk_icon_source_set_filename (isrc, filename);
   free (filename);
   gtk_icon_source_set_direction_wildcarded (isrc, TRUE);
-  gtk_icon_source_set_size_wildcarded (isrc, size == -1);
-  if (size != -1)
+  gtk_icon_source_set_size_wildcarded (isrc, (int) size == -1);
+  if ((int) size != -1)
     gtk_icon_source_set_size (isrc, size);
-  gtk_icon_source_set_state_wildcarded (isrc, state == -1);
-  if (state != -1)
+  gtk_icon_source_set_state_wildcarded (isrc, (int) state == -1);
+  if ((int) state != -1)
     gtk_icon_source_set_state (isrc, state);
   return isrc;
 }
@@ -771,16 +806,23 @@ static void ui_load_icons (void)
 {
   GtkIconFactory *ifactory = gtk_icon_factory_new ();
 
-  GtkIconSet *iset = ui_std_icon (ifactory, GXINE_MEDIA_PLAYLIST,
-				  "playlist.png");
-  if (iset)
-    gtk_icon_set_add_source (iset, make_icon_source ("playlist_16.png", -1,
-						     GTK_ICON_SIZE_MENU));
-
   ui_std_icon (ifactory, GXINE_MEDIA_SPEAKER, "speaker.png");
   ui_std_icon (ifactory, GXINE_MEDIA_SPEAKER_MUTE, "nospeaker.png");
   ui_std_icon (ifactory, GXINE_MEDIA_MARK, "/gxine.png");
   ui_std_icon (ifactory, GXINE_LOGO, "/gxine.png");
+//  ui_std_icon (ifactory, "gxine-settings-av-sync", "set-sync.svg");
+//  ui_std_icon (ifactory, "gxine-settings-hue", "set-hue.svg");
+  ui_std_icon (ifactory, "gxine-settings-saturation", "set-saturation.svg");
+  ui_std_icon (ifactory, "gxine-settings-contrast", "set-contrast.svg");
+  ui_std_icon (ifactory, "gxine-settings-brightness", "set-brightness.svg");
+  ui_std_icon (ifactory, "gxine-settings-volume", "set-volume.svg");
+//  ui_std_icon (ifactory, "gxine-settings-compressor", "set-compressor.svg");
+//  ui_std_icon (ifactory, "gxine-settings-amplifier", "set-amplifier.svg");
+#ifdef WITH_DEPRECATED
+  ui_std_icon (ifactory, "gxine-logo", "/gxine.png");
+#endif
+
+  icon_size_logo = gtk_icon_size_register ("gxine-logo-size", 64, 48);
 
   gtk_icon_factory_add_default (ifactory);
 }
@@ -856,7 +898,7 @@ static gint ui_click_clipboard_cb (GtkWidget *widget, GdkEventButton *event,
 void ui_init (void)
 {
   xine_cfg_entry_t entry;
-  int i;
+  unsigned int i;
   se_o_t *toolbars;
 
   static const ui_property_t tb_props[] = {
@@ -901,12 +943,12 @@ void ui_init (void)
 
   toolbars = se_create_object (gse, NULL, "toolbar", NULL,
 			       SE_GROUP_PROPERTIES, NULL);
-  
-  ui_create_properties (tb_props, toolbars, SE_TYPE_BOOL, NULL);
+
+  ui_create_properties (tb_props, toolbars, SE_TYPE_BOOL);
   ui_create_properties (tb_fs_props, se_find_object (gse, toolbars, "fs"),
-			SE_TYPE_BOOL, NULL);
+			SE_TYPE_BOOL);
   ui_create_properties (tb_wm_props, se_find_object (gse, toolbars, "wm"),
-			SE_TYPE_BOOL, NULL);
+			SE_TYPE_BOOL);
   ui_load_icons ();
 
   if (xine_config_lookup_entry (xine, "gui.fullscreen_toolbar", &entry))
@@ -915,7 +957,7 @@ void ui_init (void)
     fs_toolbar_visible = (entry.num_value >> 1) & 1;
   }
 
-  for (i = 0; i < sizeof (ranges) / sizeof (ranges[0]); ++i)
+  for (i = 0; i < G_N_ELEMENTS (ranges); ++i)
   {
     if (ranges[i].setting)
     {
@@ -940,7 +982,7 @@ void ui_init (void)
 
 gboolean ui_post_init (void)
 {
-  int i;
+  unsigned int i;
   gdk_threads_enter ();
   for (i = 0; i < G_N_ELEMENTS (ranges); ++i)
     if (ranges[i].param && i != Control_VOLUME)
@@ -948,3 +990,24 @@ gboolean ui_post_init (void)
   gdk_threads_leave ();
   return FALSE;
 }
+
+/* Undo button */
+
+static void key_undo_cb (GtkAccelGroup *accel, GObject *obj, guint keyval,
+			 GdkModifierType state)
+{
+  gtk_dialog_response (GTK_DIALOG (obj), GTK_RESPONSE_REJECT);
+}
+
+GtkAccelGroup *ui_add_undo_response (GtkWidget *window, GtkAccelGroup *accel)
+{
+  if (!accel)
+  {
+    accel = gtk_accel_group_new ();
+    gtk_window_add_accel_group (GTK_WINDOW (window), accel);
+  }
+  gtk_accel_group_connect (accel, GDK_Undo, 0, 1,
+			   g_cclosure_new_object (G_CALLBACK (key_undo_cb),
+						  G_OBJECT (window)));
+  return accel;
+}
