@@ -1,8 +1,8 @@
 /*
  * Copyright (C) 2001-2005 the xine-project
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
@@ -13,10 +13,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: utils.c,v 1.64 2005/11/03 22:21:31 dsalt Exp $
+ * $Id: utils.c,v 1.74 2006/04/09 14:40:12 dsalt Exp $
  *
  * needful things
  */
@@ -27,14 +27,13 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
+#include <ctype.h>
 
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <fcntl.h>
 #include <string.h>
 
-#include <pthread.h>
-
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
@@ -63,7 +62,7 @@ int ensure_path_exists (char *path, int mode)
   char *sep, *start;
 
   start = path + (*path == '/');
-  while ( (sep = strchr (start, '/')) )
+  while  ((sep = strchr (start, '/')))
   {
     int ret;
     *sep = 0;
@@ -77,21 +76,47 @@ int ensure_path_exists (char *path, int mode)
   return ensure_mkdir (path, mode); /* leave mode as is if already present */
 }
 
-char *read_entire_file (const char *mrl, int *file_size) {
+char *make_path_uri (const char *path)
+{
+  if (g_path_is_absolute (path))
+    return g_filename_to_uri (path, NULL, NULL);
 
+  if ((*path & 0xDF) < 'A' || (*path & 0xDF) > 'Z')
+    goto make_abs;
+
+  const char *tmp = path + 1;
+  for (; *tmp; ++tmp)
+    if ((*tmp != '-' && *tmp < '0') || (*tmp > '9' && *tmp < 'A') ||
+	(*tmp > 'Z' && *tmp < 'a') || *tmp > 'z')
+      break;
+  if (tmp[0] == ':' && tmp[1] == '/')
+    return NULL; /* is a URI */
+
+  make_abs:; /* is a relative path - make it absolute then a URI */
+  char *dir = g_get_current_dir ();
+  char *abspath = g_build_path (G_DIR_SEPARATOR_S, dir, path, NULL);
+  free (dir);
+  dir = g_filename_to_uri (abspath, NULL, NULL);
+  free (abspath);
+  return dir;
+}
+
+char *read_entire_file (const char *mrl, ssize_t *file_size)
+{
   char        *buf;
   struct stat  statb;
   int          fd;
-  int	       size;
+  ssize_t      size;
 
-  if (!strncasecmp (mrl, "http://", 7)) 
+  if (!strncasecmp (mrl, "http://", 7))
     return http_download (mrl, file_size);
 
   if (file_size)
     *file_size = 0;
 
-  if (stat (mrl, &statb) < 0) {
-    /*printf ("utils: cannot stat '%s'\n", mrl); */
+  if (stat (mrl, &statb) < 0)
+  {
+    /*g_print (_("utils: cannot stat '%s'\n"), mrl); */
     return NULL;
   }
 
@@ -110,7 +135,11 @@ char *read_entire_file (const char *mrl, int *file_size) {
   /* two extra bytes to ensure LF+NUL termination */
   buf = malloc (size + 2);
   if (!buf)
+  {
+    close (fd);
+    errno = ENOMEM;
     return NULL;
+  }
 
   size = read (fd, buf, size);
   if (size < 0)
@@ -135,6 +164,44 @@ char *read_entire_file (const char *mrl, int *file_size) {
   return buf;
 }
 
+char *get_config_filename (const char *leaf)
+{
+  return g_build_filename (g_get_home_dir (), ".gxine", leaf, NULL);
+}
+
+FILE *open_write (const char *file, const char *errtitle)
+{
+  char *fname = g_strconcat (file, ".new", NULL);
+  FILE *fd = NULL;
+  if ((unlink (fname) && errno != ENOENT) || !(fd = fopen (fname, "w")))
+    display_error_modal (FROM_GXINE, errtitle, _("Can't open file ‘%s’: %s"),
+			 fname, strerror (errno));
+  free (fname);
+  return fd;
+}
+
+void close_write (const char *file, FILE *fd, const char *errtitle)
+{
+  char *fname = g_strconcat (file, ".new", NULL);
+  int error;
+
+  if ((error = ferror (fd)))
+    display_error_modal (FROM_GXINE, errtitle,
+			 _("Error while writing to ‘%s’: %s"),
+			 fname, strerror (errno));
+  if (fclose (fd))
+  {
+    display_error_modal (FROM_GXINE, errtitle,
+			 _("Error when closing ‘%s’: %s"),
+			 fname, strerror (errno));
+    error = 1;
+  }
+  if (!error && rename (fname, file))
+    display_error_modal (FROM_GXINE, errtitle,
+			 _("Error when replacing ‘%s’: %s"),
+			 file, strerror (errno));
+}
+
 void window_show (GtkWidget *const widget, GtkWidget *const parent)
 {
   g_return_if_fail (widget != NULL);
@@ -179,15 +246,15 @@ void v_display_message (gxine_msg_source src, gboolean modal,
   {
   case GTK_MESSAGE_WARNING:
     if (primary)
-      fprintf (stderr, _("%s: warning: %s: %s\n"), ptitle[src], primary, msg);
+      g_printerr (_("%s: warning: %s: %s\n"), ptitle[src], primary, msg);
     else
-      fprintf (stderr, _("%s: warning: %s\n"), ptitle[src], msg);
+      g_printerr (_("%s: warning: %s\n"), ptitle[src], msg);
     break;
   case GTK_MESSAGE_ERROR:
     if (primary)
-      fprintf (stderr, _("%s: error: %s: %s\n"), ptitle[src], primary, msg);
+      g_printerr (_("%s: error: %s: %s\n"), ptitle[src], primary, msg);
     else
-      fprintf (stderr, _("%s: error: %s\n"), ptitle[src], msg);
+      g_printerr (_("%s: error: %s\n"), ptitle[src], msg);
     break;
   default:;
   }
@@ -248,13 +315,17 @@ void display_info (gxine_msg_source src, const gchar *title,
 
 void int_to_timestring (int int_time, char *string_time, gint length)
 {
+  int sign = int_time < 0;
+  int_time = abs (int_time);
   if (int_time % 1000)
-    snprintf (string_time, length, "%02d:%02d:%06.3lf",
+    snprintf (string_time, length, "%s%02d:%02d:%06.3lf",
+	      sign ? "-" : "",
 	      int_time / 3600000,
 	      (int_time / 60000) % 60,
 	      (int_time % 60000) / 1000.0);
   else
-    snprintf (string_time, length, "%02d:%02d:%02.0lf",
+    snprintf (string_time, length, "%s%02d:%02d:%02.0lf",
+	      sign ? "-" : "",
 	      int_time / 3600000,
 	      (int_time / 60000) % 60,
 	      (int_time % 60000) / 1000.0);
@@ -300,8 +371,9 @@ static void update_start_dir (GtkWidget *dbox)
   }
 }
 
-/* GTK+ >= 2.8 */
-void gtk_file_chooser_set_do_overwrite_confirmation () __attribute__ ((weak));
+#if GXINE_GTK_OLDER_THAN(2,8,0)
+WEAK (void, gtk_file_chooser_set_do_overwrite_confirmation);
+#endif
 
 gchar *modal_file_dialog (const char *title, gboolean loading, gboolean local,
 			  gboolean uri, const gchar *pattern, GtkWidget *extra,
@@ -321,7 +393,7 @@ gchar *modal_file_dialog (const char *title, gboolean loading, gboolean local,
   gtk_file_chooser_set_local_only (ch, local);
   gtk_file_chooser_set_select_multiple (ch, FALSE);
   gtk_file_chooser_set_extra_widget (ch, extra);
-  if (gtk_file_chooser_set_do_overwrite_confirmation)
+  if (GTK_HAVE (gtk_file_chooser_set_do_overwrite_confirmation, 2, 8, 0))
     gtk_file_chooser_set_do_overwrite_confirmation (ch, TRUE);
 
   set_start_dir (dbox);
@@ -364,6 +436,31 @@ GSList *modal_multi_file_dialog (const char *title, gboolean local,
   return ret;
 }
 
+void get_mrl_from_filesystem (GtkWindow *dbox, GtkEntry *entry)
+{
+  gchar *file = modal_file_dialog (_("Select file for MRL"), TRUE, TRUE, TRUE,
+				   NULL, NULL, (GtkWidget *)dbox);
+  if (file)
+  {
+    gtk_widget_grab_focus (GTK_WIDGET (entry));
+
+    const char *suffix = gtk_entry_get_text (entry);
+    if (suffix)
+    {
+      while (isspace (*suffix))
+	++suffix;
+      suffix = (*suffix && *suffix != '/') ? strchr (suffix, '#') : NULL;
+    }
+
+    char *start = g_strconcat (file, suffix, NULL);
+    gtk_entry_set_text (entry, start);
+    gtk_editable_set_position (GTK_EDITABLE (entry), 0);
+
+    free (start);
+    free (file);
+  }
+}
+
 static gboolean close_cb (GtkWidget *widget, gpointer data)
 {
   if (data)
@@ -384,7 +481,7 @@ void utils_init (void)
 }
 
 void add_table_row_items (GtkWidget *table, gint row, gboolean expand,
-			  const gchar *labeltext, int widgets, ...)
+			  const gchar *labeltext, ...)
 {
   /* Pads out last widget to fill remaining space.
    * First widget is expandable horizontally, rest aren't.
@@ -398,15 +495,17 @@ void add_table_row_items (GtkWidget *table, gint row, gboolean expand,
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row + 1, 0, 0, 2, 2);
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
 
-  va_start (ap, widgets);
-  while (widgets--)
+  va_start (ap, labeltext);
+  GtkWidget *widget = va_arg (ap, GtkWidget *);
+  while (widget)
   {
-    GtkWidget *widget = va_arg (ap, GtkWidget *);
+    GtkWidget *next = va_arg (ap, GtkWidget *);
     gtk_table_attach (GTK_TABLE (table), widget, column,
-		      widgets ? column + 1 : GTK_TABLE (table)->ncols,
+		      next ? column + 1 : GTK_TABLE (table)->ncols,
 		      row, row + 1, xattach, yattach, 2, 2);
     yattach = xattach = GTK_FILL;
     ++column;
+    widget = next;
   }
   va_end (ap);
 }
