@@ -1,8 +1,8 @@
 /*
  * Copyright (C) 2001-2005 the xine-project
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
@@ -13,16 +13,16 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: utils.h,v 1.39 2005/10/05 21:01:25 dsalt Exp $
+ * $Id: utils.h,v 1.43 2006/04/04 22:08:52 dsalt Exp $
  *
  * needful things
  */
 
 #ifndef HAVE_UTILS_H
-#define HAVE_UTILS_H 
+#define HAVE_UTILS_H
 
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
@@ -34,10 +34,18 @@ void utils_init (void);
 
 int ensure_path_exists (char *path, int mode);
 
+char *make_path_uri (const char *);
+
 /* read an entire file and ensure that the buffer is LF+NUL-terminated
  * file_size may be NULL
  */
-char *read_entire_file (const char *mrl, int *file_size);
+char *read_entire_file (const char *mrl, ssize_t *file_size);
+
+char *get_config_filename (const char *leaf);
+
+/* open or close a file, checking for errors */
+FILE *open_write (const char *name, const char *errtitle);
+void close_write (const char *name, FILE *fd, const char *errtitle);
 
 /* display message boxes
  * if fmt==NULL, first vararg is format string and window is modal
@@ -65,7 +73,7 @@ void display_info (gxine_msg_source, const gchar *, const gchar *, ...)
     _time - (_time % 1000); \
   })
 
-void int_to_timestring (gint int_time, char* string_time, gint length) ;
+void int_to_timestring (gint int_time, char* string_time, gint length);
 int parse_timestring (const char *string_time);
 
 /* Note: result of these functions needs to be freed */
@@ -74,6 +82,7 @@ gchar *modal_file_dialog (const char *title, gboolean loading, gboolean local,
 			  GtkWidget *parent);
 GSList *modal_multi_file_dialog (const char *title, gboolean local,
 				 const char *pattern, GtkWidget *parent);
+void get_mrl_from_filesystem (GtkWindow *, GtkEntry *);
 
 void window_show (GtkWidget *widget, GtkWidget *parent); /* NULL => app */
 
@@ -81,7 +90,8 @@ void window_show (GtkWidget *widget, GtkWidget *parent); /* NULL => app */
 void hide_on_delete (GtkWidget *widget, gboolean *visible);
 
 void add_table_row_items (GtkWidget *table, gint row, gboolean expand,
-			  const gchar *labeltext, int widgets, ...);
+			  const gchar *labeltext, ...)
+			  __attribute__ ((sentinel));
 
 char *asreprintf (char **str, const char *fmt, ...);
 
